/*---------------------------------------------------------------
AccelVect function by Raptorsaurus

Returns the theoretical acceleration in the x, y, z vectors for a missile or projectile.  The values returned are in m/s^2.
This version does not include the effects of lift (wings and such).  This is more for propelled missile/rocket type objects.
Since OFP is earth based, gravity will be applied using the standard 9.80665 m/s^2 (average at sea level).

The required info passed to this function is:

- Object name (the name variable for the vehicle)
- Fdrag for the object (as returned by the Fdrag function.)
- Thrust (The power moving the object if it is under power, give value in Newtons (kg*m/s^2)
- Theoretical mass for the object in kg (kilograms).  You can find these values by looking up the actual vehicles on the internet
- Pitch of craft (One can use Dschulle's or Raptorsaurus's bank and pitch function to get this, or enter a value to force the thrust at the given angle
  off horizontal.  Valid value is from -90 to + 90 with 90 being straight up and -90 being straight down.)
- Direction of Pitch (this will force the craft to apply its thrust to move in this direction.  This is a value relative to the object's
  current heading.  Valid values are +180 to -180.  Negative values will cause the object to start to move to the left of the current
  direction and Positive values will make it deflect to the right.  If you put 0 (zero) there will be no deflection in the thrust direction.
  If you put values greater than 90 or less than -90 the craft will slow down as it turns (-180 or 180 will make the craft slow down
  without turning and even go backward if the force is high enough).

Note:  All values except for Pitch and Direction must be positive, the function will determine + or -.

Example 1:

_craft = _rocket1
_Drag = 31059 (value returned by <Fdrag.sqf> for scud at 11000 m altitude travelling a 700 m/s, this is in Newtons)
_Thrust = 93163.175
_Mass = 3516.5 (mass of scud after half its fuel is used)
_Pitch = 45 (the object is pointing its nose 45 degrees above horizontal )
_Pdir = 5 (the object is pointing its nose 5 degrees to the right of it current movement direction)

_AccelVectors = [_craft, _Drag, _Thrust, _Mass, _Pitch, _Pdir] call AccelVect

The value returned will be an array of the acceleration values in the x, y, z vectors, eg:

_xa = _AccelVectors select 0
_ya = _AccelVectors select 1
_za = _AccelVectors select 2

You can apply these to the vehicle's velocity using a time based loop and the vehicle will
change direction/speed accordingly.

To get the velocity vectors in a time based loop use the following to get the x,y,z velocity
vectors.  Where _t is the time, _xa, _ya, _za are the acceleration values returned by this function
and _xv, _yv, _zv are the velocity vectors.  Note these are based on V' = V + A * T (basic physics).

_xv = _xv + (_xa * _t)
_yv = _yv + (_ya * _t)
_zv = _zv + (_za * _t)

Initialize this function by putting this in your init.sqs:
AccelVect = preprocessFile "AccelVect.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_obj","_drag","_thrust","_mass", "_tdir", "_xacel", "_yacel", "_zacel", "_drAng"];
private ["_vel","_x","_y","_z","_Ang","_Hvel","_Mag", "_dir", "_drdir", "_pitch", "_pdir"];
private ["_Tx", "_Ty", "_Tz", "_Dx", "_Dy", "_Dz", "_Th", "_Dh"];

// get the parameters

_obj = _this select 0;
_drag = abs (_this select 1);
_thrust = abs (_this select 2);
_mass = abs (_this select 3);
_pitch = _this select 4;
_pdir = _this select 5;

if (_pitch > 90) then
   {
    _pitch = 180 - _pitch;
    _pdir = _pdir - 180;
   };
if (_pitch < -90) then
   {
    _pitch = -180 - _pitch;
    _pdir = _pdir - 180;
   };

if (_pdir > 180) then {_pdir = _pdir - 360};
if (_pdir < -180) then {_pdir = _pdir + 360};

_vel = Velocity _obj;

// get the rectangular velocity vector components

_x = _vel select 0;
_y = _vel select 1;
_z = _vel select 2;

// convert to magnitued & angle format

_Hvel = sqrt(_x^2 + _y^2);

If (_Hvel == 0) then
{
  If (_z > 0) then {_Ang = 90};
  If (_z < 0) then {_Ang = -90};
  If (_z == 0) then {_Ang = 0};
};
If (_Hvel != 0)
then
{
  _Ang = Atan (_z/_Hvel);
  _Mag = _Hvel / Cos _Ang;
}
Else
{
  _Mag = abs _z;
};

// find compass direction of projectile (movement direction is not necessarily the "getDir" value)

if (_Mag > 0 && (_x != 0 || _y != 0)) then {_dir = _x atan2 _y} else {_dir = getDir _obj};
if (_dir < 0) then { _dir = _dir + 360 };

// get the thrust direction (this always the direction the vehicle is pointing not neccecarily the direction it is moving)

_tdir = getDir _obj;

// add directional deflection to the thrust direction.

_tdir = _tdir + _pdir;

// convert the direction to compass direction

if (_tdir < 0) then { _tdir = _tdir + 360 };
if (_tdir > 360) then { _tdir = _tdir - 360 };

// force of thrust in horizontal direction (x-y) plane

_Th = _thrust * cos _pitch;

_Tx = _Th * Sin _tdir;
_Ty = _Th * Cos _tdir;

// calculate vertical thrust component

_Tz= _thrust * Sin _pitch;

// calculate the direction and angle that drag will impede the motion

_drdir = _dir - 180;
if (_drdir < 0) then {_drdir = _drdir + 360};
_drAng = -_Ang;

// force of drag in horizontal direction (x-y) plane

_Dh = _drag * cos _drAng;

_Dx = _Dh * Sin _drdir;
_Dy = _Dh * Cos _drdir;

// calculate vertical drag component

_Dz= _drag * Sin _drAng;

// add the resulting vectors

_x = _Tx + _Dx;
_y = _Ty + _Dy;
_z = _Tz + _Dz;

// caculate the effect of weight (gravity) on the z vector

_z = (_mass * -9.80665) + _z ;

// calculate the acceleration in each vector

_xacel = _x/_mass;
_yacel = _y/_mass;
_zacel = _z/_mass;

// return values in array

[_xacel, _yacel, _zacel]