/*---------------------------------------------------------------
Arc function by Raptorsaurus
v. 1.0, 09/23/12

Given a center point, a radius, the arc reference direction, the arc start
angle, the arc end angle, the number of points along the arc and the arc
tilt; this function will return an array of positions along the arc path.
Both the absolute positions and the relative positions (positions relative
to the center and direction) are returned.

The required info passed to this function is:

- Center Point
- Radius
- Reference Direction (the compass heading of the arc)
- The arc start angle (-180 to 180, 0 [zero] is the arc's reference direction/high point)
- The arc end angle (-180 to 180, 0 [zero] is the arc's reference direction/high point)
- The number of points (number of evenly spaced positons desired along the arc path)
- The tilt angle (90 will make zero angle point of arc strait above center, 0 will make arc parallel to the ground)

NOTE: The start angle must be less than the end angle.

Example:

_cpos = getPos player
_rad = 50
_dir = getDir player
_sAng = -90
_eAng = 90
_num = 10
_tilt = 45

_PosInfo = [_cpos, _rad, _dir, _sAng, _eAng, _num, _tilt] call Arc

_abspos = _posInfo select 0
_relpos = _posInfo select 1

_abspos will be an array of 10 positions in the OFP world such that if one placed an object
at each position the objects would form a curved path all equal distance from a common point.
_relpos will be an array of 10 positions that are relative to the center position and the reference
direction. In this case the positions will be relative to the player.

Initialize this function by putting this in your init.sqs:
Arc = preprocessFile "Arc.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_cpos","_rad","_dir","_sAng","_eAng","_num","_tilt", "_SLref","_GLref","_ASL","_inc","_ang"];
private ["_rPosA","_X","_Y","_Z","_cnt","_pos","_aPosA","_rX","_rY","_rZ","_Gsla","_ASLcomp"];

// get the parameters

_cpos = _this select 0;
_rad = _this select 1;
_dir = _this select 2;
_sAng = _this select 3;
_eAng = _this select 4;
_num = _this select 5;
_tilt = _this select 6;

// make sure _dir is valid value

if (_dir > 360) then {_dir = _dir - 360};
if (_dir < 0) then {_dir = _dir + 360};
if (_dir == 360) then {_dir = 0};

//Find sea level altitude at _cpos

_SLref = "emptydetector" camcreate [0,0,0];
_GLref = "logic" camcreate [0,0,0];
_SLref setPos [_cpos select 0, _cpos select 1];
_GLref setPos _cpos;
_ASL = _GLref distance _SLref;

// Calculate arc points on plane normal to OFP ground
_inc = abs (_eAng - _sAng) / _num;
_ang = _sAng;
_rPosA = [];

While {_ang <= _eAng} Do 
	{
	   // calculate relative offsets in horizontal plane
	   _X = _rad * Sin _ang;
	   _Y = _rad * Cos _ang;
	   // calculate _Z and modify _Y according to _tilt
	   _Z = _Y * Sin _tilt;
	   _Y = _Y * Cos _tilt;
	   _rPosA = _rPosA + [[_X,_Y,_Z]];
	   _ang = _ang + _inc;
	};

// calculate absolute position components
_cnt = 0;
_aPosA = [];

While {_cnt < count _rPosA} Do
	{
	   _pos = _rPosA select _cnt;
	   _rX = _pos select 0;
	   _rY = _pos select 1;
	   _rZ = _pos select 2;
	   _X = (_cpos select 0) + (_rY * Sin _dir) + (_rX * Sin (_dir + 90));
	   _Y = (_cpos select 1) + (_rY * Cos _dir) + (_rX * Cos (_dir + 90));
	   _GLref setPos [_X, _Y, 0];
	   _SLref setPos [_X, _Y];
	   _Gsla = _GLref distance _SLref;
	   _ASLcomp = _ASL - _Gsla;
	   _Z = _ASLcomp + _rZ;
	   _aPosA = _aPosA + [[_X,_Y,_Z]];
	   _cnt = _cnt + 1;
	};	

deleteVehicle _GLref;
deleteVehicle _SLref;

// absolute and relative position values
[_aPosA, _rPosA]