/*---------------------------------------------------------------
IncDir function by Raptorsaurus

This function returns an incremented turn angle according to a given bearing and
increment.  The function finds the best direction to turn (clockwise or counter-
clockwise) and returns that information as well.

Pass info to the function in the following format:

_Turn = [_obj,_ang,_incval] call IncDir

The _obj is the object that you need to rotate, _ang is the angle
that you want _obj eventually to be facing and _incval will be the incremental
value in which to turn.

Here is what _Turn includes:

_heading = _Turn select 0 -- the new incremented heading for the object
_incval  = _Turn select 1 -- the same _incval entered or -_incval (negative)*

* If the shortest direction to turn to get to the desired direction is
  counter-clockwise the function will convert _incval to a negative number.
  If the degrees off course are less than _incval, then _incval will be altered
  so that the returned heading will not "overshoot" the target. 

Example:

You have a plane that is traveling NE (45 degrees) and you want to force it to
turn to the new heading of 315 degrees.  It is both unwise and looks silly if
you just impose the setDir command on the plane and have it jump instantly to
315 degrees.  At low altitudes this will make the plane crash, but even if it
does not it looks unrealistic.  So instead you can use this function in a loop
that will change the planes heading in small increments.

_ang = 315
_maxinc = .1
#loop
_Turn = [_plane,_ang,_maxinc] call IncDir
_heading = _Turn select 0
_incval = _Turn select 1
_plane setDir _heading
~.01
?(abs(_incval) >= _maxinc):goto "loop"

the plane will smoothly turn to 315 degrees and the function will know to make
it a counterclockwise rotation so you don't have to mess with lots of if then
satements and trig functions.

Initialize this function by putting this in your init.sqs:
IncDir = preprocessFile "IncDir.sqf"
---------------------------------------------------------------*/
private ["_obj","_inc","_heading","_ang","_checkval"];
private ["_ang1","_ang2","_degoff"];

_obj = _this select 0;
_ang = _this select 1;
_inc = _this select 2;

_heading = getDir _obj;

// make sure _inc is initially positive

_inc = abs _inc;

// make sure _ang is true compass heading

if (_ang <= 0) then {_ang = _ang + 360};
if (_ang > 360) then {_ang = _ang - 360};

// Find Direction Offset

_ang1 = _heading;
_ang2 = _ang;
if ((_ang1 - _ang2) > 180) then {_ang2 = _ang2 + 360};
if ((_ang2 - _ang1) > 180) then {_ang1 = _ang1 + 360};
_degoff = (_ang2 - _ang1);

if (abs _degoff < _inc) then {_inc = abs _degoff};

// convert _inc to negative if conditions are met

if (_degoff < 0) then {_inc = -_inc};

// Find new heading

_heading = _heading + _inc;

// make sure _heading is true compass heading

if (_heading <= 0) then {_heading = _heading + 360};
if (_heading > 360) then {_heading = _heading - 360};

// return values in array

[_heading,_inc]