/*---------------------------------------------------------------
IncDirtoObj function by Raptorsaurus

Returns the heading to begin turning an object toward another object in incremental
steps.

Pass info to the function in the following format:

_Turndata = [_obj1,_obj2,_incval,_offang] call IncDirtoObj

_obj1 is the object that you need to rotate, _obj2 is the object toward which you
want _obj1 to be facing, _incval will be the maximum incremental value in which to turn, and
_offang is the offset angle.  If you want _obj1 to head straight toward _obj2 then _offang should
be zero.  If you want it to face to the left of _obj then _offang should be negative (up to -180).
If you want _obj1 to face to the right of _obj2 then _offang should be positive (up to 180).
Negative 180 or postive 180 will actually make _obj1 turn in the opposite direction of _obj2.

_Turndata will contain the new heading for _obj1 to turn toward _obj2 in _incval increments,
the bearing toward _obj2 and -_incval (negative value indicating shortest rotation is CCW)
or _incval (positive value indicating shortestrotaion is CW).  If the degrees off course
are less than the maximum _incval, then _incval will be altered so that the returned heading
will not "overshoot" the target. 

Example

_Turndata = [_plane1,_plane2,.5,0] call IncDirtoObj

_newheading = _Turndata select 0 (incremented heading toward target)
_bearing = _Turndata select 1 (full direction toward target)
_inc = _Turndata select 2 (increment value)

If the shortest rotation for the plane1 to face plane2 is to the left then
_inc will be -.5 otherwise it will be .5.  The _newheading will be the plane's
present heading + _inc to bring the plane towards _bearing which is the exact direction
to _plane2.  If the plane is less than .5 degrees off course of _bearing, then _newheading
will be the same as _bearing and _inc will be the value needed to bring plane 1 to the
_newheading.

Initialize this function by putting this in your init.sqs:
IncDirtoObj = preprocessFile "IncDirtoObj.sqf"
---------------------------------------------------------------*/
private ["_obj1","_obj2","_inc","_heading","_bearing","_checkval"];
private ["_ang1","_ang2","_degoff","_offang", "_Dx", "_Dy"];

_obj1 = _this select 0;
_obj2 = _this select 1;
_inc = _this select 2;
_offang = _this select 3;

// make sure _inc is initially positive

_inc = abs _inc;

// make sure _offang is in range -180 to 180

if (_offang < -180) then {_offang = -180 - _offang};
if (_offang > 180) then {_offang = 180 - _offang};

// Get current heading of _obj1

_heading = getDir _obj1;

// Get bearing to _obj2

_Dx = (getpos _obj2 select 0) - (getpos _obj1 select 0);
_Dy = (getpos _obj2 select 1) - (getpos _obj1 select 1);
if (_Dy == 0) then {_Dy = .0000001};
_bearing = _Dx atan2 _Dy;

// Modifiy _bearing with _offang

_bearing = _bearing + _offang;

// Make sure _bearing is valid compass direction

if (_bearing < 0) then {_bearing = _bearing + 360};
if (_bearing > 360) then {_bearing = _bearing - 360};

// Find Direction Offset

_ang1 = _heading;
_ang2 = _bearing;
if ((_ang1 - _ang2) >= 180) then {_ang2 = _ang2 + 360};
if ((_ang2 - _ang1) > 180) then {_ang1 = _ang1 + 360};
_degoff = (_ang2 - _ang1);

if (abs _degoff < _inc) then {_inc = abs _degoff};

// convert _inc to negative if conditions are met

if (_degoff < 0) then {_inc = -_inc};

// Add _inc to _heading and make sure it is valid compass direction

_heading = _heading + _inc;
if (_heading < 0) then {_heading = _heading + 360};
if (_heading > 360) then {_heading = _heading - 360};

// return values in array

[_heading,_bearing,_inc]