/*---------------------------------------------------------------
Flift function by Raptorsaurus

Returns the force of the lift and x, y and z vectors of the lift force based on the craft's bank and pitch.  The values returned are in miliNewtons
(mN) or g*m/s^2.

The required info passed to this function is:

- Object name (the name variable for the vehicle)
- Lift Coefficient for the object (infomation for this value can be looked up for various aircraft and missiles used in OFP).
- Lift area of the object (in square meters - the total area of the structures on the craft providing lift in square meters m^2).
- Temperature at sea level (for standard OFP islands it will be around 5 to 7 deg C at night and 10 to 20 deg C at day depending
   on the season and the weather you choose in you mission.  For winter islands it will be below 0 C, for Vietnam islands it will be about
   30 to 40 C, you get the idea).
- Pitch of craft (use the bank & pitch function by Raptorsaurus or Dschulle)
- Bank of craft (use the bank & pitch function by Raptorsaurus or Dschulle)

Example:

_craft = _plane1
_C = .095
_A = 2.5
_T = 20
_P = -35
_B = 15

_Lift = [_craft, _C, _A, _T, _P, _B] call FLift

_liftForce = _Lift select 0
_ 

Initialize this function by putting this in your init.sqs:
Fdrag = preprocessFile "Fdrag.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_obj","_C","_A","_T0","_V","_Ro","_P", "_L", "_T" ,"_h", "_Hvel"];
private ["_x","_y", "_z", "_SLref","_Hdis","_Dis","_Ang", "_Fdrag", "_st"];

// get the parameters

_obj = _this select 0;
_C = _this select 1;
_A = _this select 2;
_T0 = _this select 3;

// Convert _T0 to Kelvin

_T0 = _T0 + 273.15;

//Get speed and altitude above sea level

_V = Velocity _obj;

// get the rectangular velocity vector components

_x = _V select 0;
_y = _V select 1;
_z = _V select 2;

// convert to magnitued & angle format

_Hvel = sqrt(_x^2 + _y^2);

If (_Hvel == 0) then
{
  If (_z > 0) then {_Ang = 90};
  If (_z < 0) then {_Ang = -90};
  If (_z == 0) then {_Ang = 0};
};
If (_Hvel != 0)
then
{
  _Ang = Atan (_z/_Hvel);
  _V = _Hvel / Cos _Ang;
}
Else
{
  _V = abs _z;
};

//Find _h (altitude or height above sea level)

_SLref = "emptyDetector" camCreate [0,0,0];
_SLref setPos [getPos _Obj select 0, getPos _Obj select 1];
_h = _Obj distance _SLref;
deleteVehicle _SLref;

// find air pressure at _h

_P = 101325 * ((1 + (-.0065 * _h / _T0))^5.255876);
if (_P < 0) then {_P = 0};

// find temperature at _h;

_T = _T0 + (-.0065 * _h);

// Find air density at _h

_Ro = _P / (8.31432 * _T);
if (_Ro < 0) then {Ro = 0};

// Finally, caculate the Fdrag at _h altitude and _V velocity with the other given parameters
// Fdrag is calculated as a postitive number allowing other scripts/functions to determine the sign.

_Fdrag = Abs (.5 * _C * _Ro * _A * (_V^2));
_st = format ["%1",_Fdrag];
If (_st == "-1.#IND" || _st == "1.#QNAN") then {_Fdrag = 0};

_Fdrag