/*---------------------------------------------------------------
IncElev function by Raptorsaurus

This function returns an incemented climb/dive angle according to a given
elevation angle and increment.  It can be used to incrementally redirect an
aircraft to a new elevation.

Pass info to the function in the following format:

_Elevation = [_obj,_ang,_incval] call IncElev

The _obj is the object that you need to elevate, _ang is the eventual angle of
climb/dive desired for the objcet.  Valid values are from +90 to -90 deg.  Where
+90 is straight up and -90 is straight down.  The _incval will be the incremental
value in which to alter the climb/dive angle.

Here is what _Elevation includes:

_elevang = _Elevation select 0 -- the new incremented climb/dive for the object
_incval  = _Elevation select 1 -- the same _incval entered or -_incval*

* If the new elevation angle is less than the objects starting clime/dive angle
  then _incval will become a negative number.  If the angle required to get to the
  new angle is less than _incval, then _incval will be adjusted accordingly.

Example:

You have a plane that is flying at 35 feet in level flight, but you want to
force it to climb at a 30 deg angle.  Use this function along with VelToAng in a loop.

_ang = 30
_maxinc = .05
_max = (speed _plane) * 1000 / 3600  ;converts speed from km/h to m/s
_min = _max - 5
_cor = -.0109 * _ang/3
#loop
_Elevation = [_plane,_ang,_maxinc] call IncElev
_elevang = _Elevation select 0
_incval = _Elevation select 1

_Vel = [_plane,_elevang,_cor,_min,_max] call VeltoAng
_plane setVelocity _Vel
?(abs(_incval) >= _maxinc):goto "loop"

the plane will smoothly transition to a climb elevation of 30 degrees.

Initialize this function by putting this in your init.sqs:
IncElev = preprocessFile "IncElev.sqf"
---------------------------------------------------------------*/

// declare private variables

private ["_inc","_finang","_vel","_x","_y","_z","_Hvel","_Ang","_elevang"];

// get object velocity, desired ang of climb/dive and increment

_vel = Velocity (_this select 0);
_finang = _this select 1;
_inc = _this select 2;

// make sure _inc is initially a positive number

_inc = abs _inc;

// get individual velocity components

_x = _vel select 0;
_y = _vel select 1;
_z = _vel select 2;

// find object's present climb/dive angle from the velocity components

_Hvel = sqrt(_x^2 + _y^2);

If (_Hvel == 0) then
{
  If (_z > 0) then {_Ang = 90};
  If (_z < 0) then {_Ang = -90};
  If (_z == 0) then {_Ang = 0};
};
If (_Hvel != 0) then {_Ang = Atan (_z/_Hvel)};

// detemine if _inc is greater than the angle off of desired, if so make _inc the exact needed value

if (abs _inc > abs(_finang - _Ang)) then {_inc = abs(_finang - _Ang)};

// determine if _inc needs to be positive or negative

if (_finang < _Ang) then {_inc = -_inc};

// Now calculate incremented angle

_elevang = _Ang + _inc;

// return values in array

[_elevang,_inc]
