/*---------------------------------------------------------------
IncElevtoObj function by Raptorsaurus

This function returns an incemented climb/dive angle required for a flying object to
get to the same sea level altitude as a second oject over a given distance.

Pass info to the function in the following format:

_Elevation = [_obj1, _obj2,_dis,_incval,_altoff] call IncElevtoObj

The _obj1 is the object that you need to elevate, _obj2 is the object at which
elevation you want _obj1 to be go to, _dis is the distance over which to get to
the elevation of _obj2.  The _incval will be the incremental value in which to
alter the climb/dive angle and _altoff is the altidue offset (so you can adjust
_plane1 to fly higher or lower than _plane2).

Here is what _Elevation includes:

_elevang = _Elevation select 0 -- the new incremented climb/dive for the _obj1
_ang = _Elevation select 1 -- the final climb dive angle to get to altitude
_incval  = _Elevation select 2 -- the same _incval entered or -_incval*

* If the new elevation angle is less than the objects starting clime/dive angle
  then _incval will become a negative number.  If the angle required to get to the
  new angle is less than _incval, then _incval will be adjusted accordingly.

Example:

You have a plane that is flying at 35 feet in level flight, but you want to
force it to the same elevation as a second plane.  Use this function along
with VelToAng in a loop.

_dis = (_plane1 distance _pane2) - 50
_maxinc = .05
_altoffset = 20
_max = (speed _plane1) * 1000 / 3600  ;converts speed from km/h to m/s
_min = _max - 5
#loop
~.0000001
_Elevation = [_plane1, _plane2, _dis, _maxinc,_altoffset] call IncElevtoObj
_elevang = _Elevation select 0
_incval = _Elevation select 1
_cor = -.0109 * _elevang/3
_Vel = [_plane1,_elevang,_cor,_min,_max] call VeltoAng
_plane setVelocity _Vel
?(abs(_incval) >= _maxinc):goto "loop"

the _plane1 will smoothly transition into a climb to an altitude 20 m above _plane2.
To hold _plane1 at this elevation you would need a second loop
to repeat this loop when _plane one either gets to high above or too low below
_plane2.

Initialize this function by putting this in your init.sqs:
IncElevtoObj = preprocessFile "IncElevtoObj.sqf"
---------------------------------------------------------------*/

// declare private variables

private ["_obj1", "_obj2","_inc","_Ang","_vel","_x","_y","_z","_Hvel","_Dis","_elevang","_SLref"];
private ["_climbdis","_Hdis","_ASL1","_ASL2","_st","_Dalt","_finang","_altoff"];

_obj1 = _this select 0;
_obj2 = _this select 1;
_climbdis = _this select 2;
_inc = _this select 3;
_altoff = _this select 4;

// make sure _inc is initially a positive number

_inc = abs _inc;

// Put Sea level reference

_SLref = "logic" camCreate [0,0,0];

// get the position of _obj1

_Pos = getPos _obj1;
_x = _Pos select 0;
_y = _Pos select 1;

//Calculate ASL of _obj1

_Hdis = sqrt(_x^2 + _y^2);
_Dis = _obj1 distance _SLref;
_Ang = Acos (_Hdis/_Dis);
_ASL1 = _Hdis * Tan _Ang;
_st = format ["%1",_ASL1];
If (_st == "-1.#IND" || _st == "1.#QNAN") then {_ASL1 = 0};

// get the position of _obj2

_Pos = getPos _obj1;
_x = _Pos select 0;
_y = _Pos select 1;

//Calculate ASL of _obj2

_Hdis = sqrt(_x^2 + _y^2);
_Dis = _obj2 distance _SLref;
_Ang = Acos (_Hdis/_Dis);
_ASL2 = _Hdis * Tan _Ang;
_st = format ["%1",_ASL2];
If (_st == "-1.#IND" || _st == "1.#QNAN") then {_ASL2 = 0};

//Delete _SLref

deleteVehicle _SLref;

//Find altitude difference and angle need to get _obj1 to that altitude
//over the given distance

_Dalt = (_ASL2 - _ASL1) + _altoff;
_finang = Atan (_Dalt/_climbdis);

// get individual velocity components of _obj1

_vel = Velocity _obj1;

_x = _vel select 0;
_y = _vel select 1;
_z = _vel select 2;

// find _obj1 present climb/dive angle from the velocity components

_Hvel = sqrt(_x^2 + _y^2);

If (_Hvel == 0) then
{
  If (_z > 0) then {_Ang = 90};
  If (_z < 0) then {_Ang = -90};
  If (_z == 0) then {_Ang = 0};
};
If (_Hvel != 0) then {_Ang = Atan (_z/_Hvel)};

// detemine if _inc is greater than the angle off of desired, if so make _inc the exact needed value

if (abs _inc > abs(_finang - _Ang)) then {_inc = abs(_finang - _Ang)};

// determine if _inc needs to be positive or negative

if (_finang < _Ang) then {_inc = -_inc};

// Now calculate incremented angle

_elevang = _Ang + _inc;

// return values in array

[_elevang, _finang, _inc]