/*---------------------------------------------------------------
IncVeltoTarg function by Raptorsaurus

Returns the velocity and directional information required to incrementally
alter a projectile object's trajectory toward a target object.  The infomation
returned is:  The new velocity after the incremental angle ajustment is applied,
the relative direction to the target in the horizontal plane (-180 to 180; postitive value
indicates projectile must turn left to head towards the target), the relative angle
in the vertical plane (-90 to 90; a negative value indicates the target is below the
projectile) and the relative velocity magnitude toward the target in m/s (a negative
value indicates the projecitle is falling away from the target, a positive value indicates
that the projectile is closing on the target).

This function will calculate the required information based on direction and speed of the
target and the direction and speed of the projectile and the distance between the projectile
and target (i.e. it finds the information to reach the spot where the target WILL be at
impact time, NOT where the target is now).

The required info passed to this function is:

Projectile object name
Target object name
Maximum directional increment
Maximum elevation angle increment
Speed modifier (postive to speed projectile, negative to slow it, 0 to not change it)

Example:

_proj = _missile
_targ = _tank1
_dirinc = 1
_anginc = 10
_spd = 0

_Interceptdata = [_proj, _targ, _dirinc, _anginc, _spd] call IncVeltoTarg

_vel = _Interceptdata select 0
_diroff = _Interceptdata select 1
_angoff = _Interceptdata select 2
_relspd = _Interceptdata select 3

_vel is the resulting veloctiy after the _dirinc  and _anginc is applied to the direction
         and climb/dive angle of the projectile (the projectile's speed is maintained).
         if _dirinc/_anginc is greater than the actual direction/angle off course then the actual
         value applied will only be that necessary to direct the projectil toward the
         target.
_diroff is the current relative degree the pojectile's trajectory is from a collision
              course with the target in the horizontal plane.  This can be used to mointor
              the progess of a script loop (i.e. to exit the loop when a desired value is achieved).
_angoff is the current relative degree the pojectile's trajectory is from a collision
               course with the target in the vertical plane (climb/dive angle).  This can be used to
               mointor the progess of a script loop (i.e. to exit the loop when a desired value is achieved).
_relspd is the relative speed of the projectile toward the target in m/s.  This can be used to
                determine if the projectile is going too slow to ever hit the target.  If it is a negative value
                then the target is "outrunning" the projectile and impact is impossible unless the target
                turns toward the projectile or the projectile speed is increased.

Initialize this function by putting this in your init.sqs or at the start of the script(s) using this function.:
IncVeltoTarg = preprocessFile "IncVeltoTarg.sqf"
---------------------------------------------------------------*/
// declare private variables
private ["_Pvel","_Ppos","_XvelP","_YvelP","_ZvelP","_XposP","_YposP","_ZposP","_Tdir", "_RDir", "_Ref2"];
private ["_Tvel","_Tpos","_XvelT","_YvelT","_ZvelT","_XposT","_YposT","_ZposT","_Rspd", "_anginc"];
private ["_Proj","_Targ","_Dir","_Ang","_Pspd","_Dis","_Tint", "_incDir", "_incAng", "_dirinc","_PDir","_RAng"];
private ["_Ref","_Pang","_TdeltaX","_TdeltaY","_TdeltaZ","_Hvel","_x","_y","_z","_vel","_Tang","_Tspd","_THvel","_spd"];

_Proj = _this select 0;
_Targ = _this select 1;
_dirinc = abs (_this select 2);
_anginc = abs (_this select 3);
_spd = _this select 4;

// get the velocity & pos

_Pvel = Velocity _Proj;
_Ppos = getPos _Proj;
_Tvel = Velocity _Targ;
_Tpos = getPos _Targ;

// get the rectangular velocity vector components

_XvelP = (_Pvel select 0);
_YvelP = (_Pvel select 1);
_ZvelP = (_Pvel select 2);

_XvelT = (_Tvel select 0);
_YvelT = (_Tvel select 1);
_ZvelT = (_Tvel select 2);

// find projectile speed and movement direction from velocity vectors
// (this is more reliable than using "getDir" and "Speed" commands)

_Hvel = sqrt(_XvelP^2 + _YvelP^2);

If (_Hvel == 0) then
{
  If (_ZvelP > 0) then {_Pang = 90};
  If (_ZvelP < 0) then {_Pang = -90};
  If (_ZvelP == 0) then {_Pang = 0};
};
If (_Hvel != 0)
then
{
  _Pang = Atan (_ZvelP/_Hvel);
  _Pspd = _Hvel / Cos _Pang;
}
Else
{
  _Pspd = abs _ZvelP;
};

// find compass direction of projectile (movement direction is not necessarily the "getDir" value)

if (_XvelP != 0 || _YvelP != 0) then {_PDir = _XvelP atan2 _YvelP} else {_PDir = getDir _Proj};
if (_PDir < 0) then { _PDir = _PDir + 360 };

// find target speed and movement direction from velocity vectors
// (this is more reliable than using "getDir" and "Speed" commands)

_THvel = sqrt(_XvelT^2 + _YvelT^2);

If (_THvel == 0) then
{
  If (_ZvelT > 0) then {_Tang = 90};
  If (_ZvelT < 0) then {_Tang = -90};
  If (_ZvelT == 0) then {_Tang = 0};
};
If (_THvel != 0)
then
{
  _Tang = Atan (_ZvelT/_THvel);
  _Tspd = _THvel / Cos _Tang;
}
Else
{
  _Tspd = abs _ZvelT;
};

// find compass direction of target (movement direction is not necessarily the "getDir" value)

if (_XvelT != 0 || _YvelT != 0) then {_TDir = _XvelT atan2 _YvelT} else {_TDir = getDir _Targ};
if (_TDir < 0) then { _TDir = _TDir + 360 };

// get the current position components

_XposP = (_Ppos select 0);
_YposP = (_Ppos select 1);

_XposT = (_Tpos select 0);
_YposT = (_Tpos select 1);

// Correct z values to sea level

_Ref = "emptyDetector" camCreate [0,0,0];
_Ref setPos [_XposP, _YposP];
_ZposP = _Proj distance _Ref;

_Ref setPos [_XposT, _YposT];
_ZposT = _Targ distance _Ref;

// find the speed of the projectile relative to the target using delta distance (assuming constant velocity over next 1 second)

_Ref2 = "emptyDetector" camCreate [0,0,0];
_Dis = _Proj distance _Targ;
_x = _XvelP + _XposP;
_y = _YvelP + _YposP;
_z = _ZvelP + _ZposP;
_Ref setPos [_x, _y, _z];
_x = _XvelT + _XposT;
_y = _YvelT + _YposT;
_z = _ZvelT + _ZposT;
_Ref2 setPos [_x, _y, _z];
_Rspd = _Dis - (_Ref distance _Ref2);

// Calc time to intercept at projectile's relative speed and distance to target (assumes positive relative speed)

if ((_Rspd - _Rspd) != 0 || _Rspd == 0) then {_Rspd = .000001};
_Tint = _Dis / abs _Rspd;

// Calc target position change in time to intercept

_TdeltaX = _Tint * _XvelT;
_TdeltaY = _Tint * _YvelT;
_TdeltaZ = _Tint * _ZvelT;

_XposT = _XposT + _TdeltaX;
_YposT = _YposT + _TdeltaY;
_ZposT = _ZposT + _TdeltaZ;

// Find Projectile's relative direction and angle to intercept point

// Direction and distance to projected target location in x,y plane

if ((_YposT - _YposP) !=0 || (_XposT - _XposP) != 0)
then 
   {_Dir = (_XposT - _XposP) Atan2 (_YposT - _YposP)}
else
   {_Dir = 0};
if (_Dir < 0) then { _Dir = _Dir + 360};

_Dis = Sqrt((_XposP - _XposT)^2 + (_YposP - _YposT)^2);

// Find relative direction to target

_RDir = _Dir - _PDir;
if (_RDir > 180) then {_RDir = (_RDir - 360)};
if (_RDir < -180) then {_RDir = (_RDir + 360)};

//Incremented direction to the target

_incDir = _dirinc;
if (abs _RDir < _dirinc) then {_incDir = abs _RDir};
if (_RDir < 0) then {_incDir = -_incDir};
_Dir = _PDir + _incDir;

//to prevent div by zero error if _proj is zero distance from _trag

if (_Dis < .000001) then {_Dis = .000001};

// Relative angle to target in z plane

_RAng = (Atan((_ZposT - _ZposP)/_Dis)) - _Pang;

//Incremented angle to target

_incAng = _anginc;
if (abs _RAng < _anginc) then {_incAng = abs _RAng};
if (_RAng < 0) then {_incAng = -_incAng};
_Ang = _Pang + _incAng;

//Add speed modifier to projectile speed

_Pspd = _Pspd + _spd;

// calculate the projected horizontal velocity component at current speed and _Ang

_Hvel = _Pspd * Cos _Ang;

// calculate projected velocity components in horizontal plane from _Hvel and _Dir

_x = _Hvel * Sin _Dir;
_y = _Hvel * Cos _Dir;

// calculate vertical velocity component

_z = _Pspd * Sin _Ang;
_vel = [_x,_y,_z];
deleteVehicle _Ref;
deleteVehicle _Ref2;
[_vel, _RDir, _RAng, _Rspd]