/*---------------------------------------------------------------
Intercept function by Raptorsaurus

Returns the Direction, Climb/Dive angle and the rectangular velocity
components required for a moveing object (projectile) to intercept another
moving object (target) at the current speeds of the objects, the current
direction of the target and the initial position of the projectile (assuming
projectile is heading in general direction of the target and has higher rate of
speed than target).

The required info passed to this function is:

Projectile object name
Target object name
Lead Factor     (positive to have projectile fly in front, negative to fly behind
                (for warning "bow shots" or for "forced" aircraft formations)
Leeway Factor   (positive to have projectile fly to right, negative to fly to left
                (for warning "bow shots" or for "forced" aircraft formations)
Altitude Factor (positive to have projectile fly over, negative to fly under
                (for warning "bow shots" or for "forced" aircraft formations)

Example:

_proj = _missile
_targ = _tank1
_leddfact = 0
_leefact = 0
_altfact = 0

_Interceptdata = [_proj,_targ,_ledfact,_leefact,_altfact] call Intercept

_Interceptdir = _Interceptdata select 0
_Interceptang = _Interceptdata select 1
_Interceptvel = _Interceptdata select 2

_interceptdir is the compass heading direction to the projected intercept point.
_interceptang is the climb/dive angle required to intercept the target.
_Interceptvel is an array of the _x,_y,_z velocity setting required to put the
projectile on a collision course with the target.

These three pieces of information can then be used in a loop that alters the
projectile's direction and velocity while continuously updating the intercept
data as it closes on the target for devestating accuracy, or if distance and
altitude factors are other thatn zero, to make a tight aircraft formation!!

Initialize this function by putting this in your init.sqs:
Intercept = preprocessFile "Intercept.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_Pvel","_Ppos","_XvelP","_YvelP","_ZvelP","_XposP","_YposP","_ZposP","_Tdir","_leefact","_leefactX"];
private ["_Tvel","_Tpos","_XvelT","_YvelT","_ZvelT","_XposT","_YposT","_ZposT","_ledfactX","_ledfactY"];
private ["_Proj","_Targ","_Dir","_Ang","_Pspd","_Dis","_Tint","_cnt","_iter","_ledfact","_altfact"];
private ["_HrefdisP","_refdisP","_refangP","_Ref","_HrefdisT","_refdisT","_refangT","_Pang","_leefactY"];
private ["_TdeltaX","_TdeltaY","_TdeltaZ","_Hvel","_x","_y","_z","_vel","_Tang","_Tspd","_THvel","_Dx","_Dy"];

_Proj = _this select 0;
_Targ = _this select 1;
_ledfact = _this select 2;
_leefact = _this select 3;
_altfact = _this select 4;

// get the velocity & pos

_Pvel = Velocity _Proj;
_Ppos = getPos _Proj;
_Tvel = Velocity _Targ;
_Tpos = getPos _Targ;

// get the rectangular velocity vector components

_XvelP = (_Pvel select 0);
_YvelP = (_Pvel select 1);
_ZvelP = (_Pvel select 2);

_XvelT = (_Tvel select 0);
_YvelT = (_Tvel select 1);
_ZvelT = (_Tvel select 2);

// find projectile speed and movement direction from velocity vectors
// (this is more reliable than using "getDir" and "Speed" commands)

_Hvel = sqrt(_XvelP^2 + _YvelP^2);

If (_Hvel == 0) then
{
  If (_ZvelP > 0) then {_Pang = 90};
  If (_ZvelP < 0) then {_Pang = -90};
  If (_ZvelP == 0) then {_Pang = 0};
};
If (_Hvel != 0)
then
{
  _Pang = Atan (_ZvelP/_Hvel);
  _Pspd = _Hvel / Cos _Pang;
}
Else
{
  _Pspd = abs _ZvelP;
};

// find compass direction of projectile (movement direction is not necessarily the "getDir" value)

if (_Hvel != 0) then {_Dir = _XvelP atan2 _YvelP} else {_Dir = getDir _Proj};
if (_Dir < 0) then { _Dir = _Dir + 360 };

// find target speed and movement direction from velocity vectors
// (this is more reliable than using "getDir" and "Speed" commands)

_THvel = sqrt(_XvelT^2 + _YvelT^2);

If (_THvel == 0) then
{
  If (_ZvelT > 0) then {_Tang = 90};
  If (_ZvelT < 0) then {_Tang = -90};
  If (_ZvelT == 0) then {_Tang = 0};
};
If (_THvel != 0)
then
{
  _Tang = Atan (_ZvelT/_THvel);
  _Tspd = _THvel / Cos _Tang;
}
Else
{
  _Tspd = abs _ZvelT;
};

// find compass direction of target (movement direction is not necessarily the "getDir" value)

if (_THvel != 0) then {_TDir = _XvelT atan2 _YvelT} else {_TDir = getDir _Targ};
if (_TDir < 0) then { _TDir = _TDir + 360 };

// Calculate the x and y components of the _ledfact and _leefact

_ledfactX = _ledfact * Sin _Tdir;
_ledfactY = _ledfact * Cos _Tdir;
_leefactX = _leefact * Sin (_Tdir + 90);
_leefactY = _leefact * Cos (_Tdir + 90);

// get the current position components

_XposP = (_Ppos select 0);
_YposP = (_Ppos select 1);
_ZposP = (_Ppos select 2);

_XposT = (_Tpos select 0);
_YposT = (_Tpos select 1);
_ZposT = (_Tpos select 2);

// Correct z values to sea level

_Ref = "emptyDetector" camCreate [0,0,0];
_Ref setPos [_XposP, _YposP];
_ZposP = _Proj distance _Ref;

_Ref setPos [_XposT, _YposT];
_ZposT = _Targ distance _Ref;

// Calc time to intercept at projectile's current speed and distance to target

_Dis = _Proj distance _Targ;
if ((_Pspd - _Pspd) != 0 || _Pspd == 0) then {_Pspd = .000001};
_Tint = _Dis / _Pspd;

// Calc target position change in time to intercept and adjust for _ledfact, _leefact
// and _altfact.

_TdeltaX = _Tint * _XvelT;
_TdeltaY = _Tint * _YvelT;
_TdeltaZ = _Tint * _ZvelT;

_XposT = _XposT + _TdeltaX + _ledfactX + _leefactX ;
_YposT = _YposT + _TdeltaY + _ledfactY + _leefactY;
_ZposT = _ZposT + _TdeltaZ + _altfact;

// Find Projectile's direction and angle to intercept point

// Direction and distance to projected target location in x,y plane

_Dx = (_XposT - _XposP);
_Dy = (_YposT - _YposP);
if (_Dy == 0) then {_Dy = .0000001};
_Dir = _Dx Atan2 _Dy;
if (_Dir < 0) then { _Dir = _Dir + 360 };

_Dis = Sqrt((_XposP - _XposT)^2 + (_YposP - _YposT)^2);
if (_Dis == 0) then {_Dis = .0000001};

// Angle to target in _dis,z plane

_Ang = Atan((_ZposT - _ZposP)/_Dis);

// Find velocity vectors to intercept from direction, angle and projectile speed.

// calculate the projected horizontal velocity component at current speed and desired _ang

_Hvel = _Pspd * Cos _ang;

// calculate projected velocity components in horizontal plane

_x = _Hvel * Sin _Dir;
_y = _Hvel * Cos _Dir;

// calculate vertical velocity component

_z = _Pspd * Sin _ang;

_vel = [_x,_y,_z];

deleteVehicle _Ref;

[_Dir,_Ang,_vel]