/*---------------------------------------------------------------
LineOfSight function by Raptorsaurus
v. 1.0

Determines if a straight line path from one point to another will clear the ground, obstacles and forests.

The required info passed to this function is:

- Starting postion [x, y, z]
- Final destination position [x, y, z]
- minimum vertical clearance (how much below the path must the object/ground/forest be to consider it a clear path).
- path width. The width of the path.
- slop. The buffer zone around the destination position (to prevent objects near the final position from causing
	a return of false when those objects may themselves be the desired targets). 
-excluded. Optional parameter consisting of item types to exclude as potential obstructions to the line of sight.

Example:

_spos = [getPos shooter select 0, getPos shooter select 1, (getPos shooter select 2) + 1.6]
_epos = [getPos target select 0, getPos target select 1, (getPos target select 2) + 2]
_height = .5
_width = 5
_slop = 2

_los = [_spos, _epos, _height, _width, _slop] call LineOfSight

This will return true if there is a 5 m wide line of sight that clears the ground, objects and forests by at least
.5 meters from the _spos to the _tpos. If any portion of the 5 m wide path comes closer than .5 m to an object
_los will be false.

Initialize this function by putting this in your init.sqs:
LineOfSight = preprocessFile "LineOfSight.sqf"

For debugging:
if (! _los) then
{
_f = "fire" createVehicle [0,0,0];
_f inflame true;
_f setPos (getPos _obj);
};
---------------------------------------------------------------*/

// declare private variables
private ["_spos","_epos","_clrH","_inc","_Zref","_Pref1","_Zspos","_Zepos","_los","_rad","_rinc","_ainc","_objA"];
private ["_Hdis","_x","_y","_z","_dis","_fdis","_ang","_dir","_Dx","_Dy","_Pref2","_dang","_radcnt","_tpos","_Obj","_bad"];
private ["_oH","_oD","_wf","_doff","_af","_oRx","_oRy","_ang1","_ang2","_oZ","_typ","_selfDis","_slop","_fH","_exc"];

// get the parameters

_spos = _this select 0;
_epos = _this select 1;
_clrH = _this select 2;
_width = _this select 3;
_slop = _this select 4;
_exc = [];
if (count _this > 5) then {_exc = _this select 5};

_Zref = "emptydetector" createVehicle [0,0,0];
_Pref1 = "logic" createVehicle [0,0,0];
_Pref2 = "logic" createVehicle [0,0,0];
_los = true;
_objA = [];
_bad = ["SmokeSource","Track","Mark","ObjectDestructed","Explosion","Crater","CraterOnVehicle","Slop","Smoke","DynamicSound"];
_bad = _bad + _exc;

// get the distance to the final position

_Pref1 setPos _spos;
_Pref2 setPos _epos;
_fdis = _Pref1 distance _Pref2;
if (_fdis == 0) then {_fdis = .0000001};

// calculate elevation angle to position (referenced to sea level)

_Zref setPos [_spos select 0, _spos select 1];
_Zspos = _Pref1 distance _Zref;
_Zref setPos [_epos select 0, _epos select 1];
_Zepos = _Pref2 distance _Zref;
_ang = Asin ((_Zepos - _Zspos) / _fdis);
_Pref1 setPos [0,0,0];

// get direction to position

_Dx = (_epos select 0) - (_spos select 0);
_Dy = (_epos select 1) - (_spos select 1);
if (_Dy == 0) then {_Dy = .0000001};
_dir = _Dx atan2 _Dy;
if (_dir < 0) then {_dir = _dir + 360};

// optimize check increments

_rad = (_width / 2) + 36;
_rinc = _width / 2;
if (_rinc <= 0) then {_rinc = 1};
if (_rinc > 5) then {_rinc = 5; _rinc = (_width / 2) / (((_width / 2) / _rinc) - (((_width / 2) / _rinc) % 1))};
_inc = 5;
if (_fdis > 500) then {_inc = _fdis / 100};
_inc = _fdis / ((_fdis/_inc) - ((_fdis/_inc) % 1));

// check along the path for objects coming withing _clrH of the path

_dis = _inc;
While {_dis < _fdis && _los}
Do
	{
	_Hdis = _dis * cos _ang;
	_x = (_spos select 0) + (_Hdis * Sin _dir);
	_y = (_spos select 1) + (_Hdis * Cos _dir);
	_radcnt = 0;
	While {_radcnt <= _rad && _los}
	Do
		{
		_ainc = 180;
		_dang = _dir - 90;
		While {_dang <= (_dir + 90) && _los}
		Do
      		{
			_tpos = [(_x + (_radcnt * Sin _dang)), (_y + (_radcnt * Cos _dang))];
			_Zref setPos [_tpos select 0, _tpos select 1];
			_Pref2 setPos [_tpos select 0, _tpos select 1, 0];
			_z = (_Zspos + (_dis * sin _ang)) - (_Zref distance _Pref2);
			_Zref setPos [0,0,0];
			_Pref2 setPos [0,0,0];
			if (_radcnt > (_width / 2)) then {_af = 18} else {_af = 4};
			if (_z > _af) then {_af = _z};
			_obj = nearestObject [_tpos select 0,_tpos select 1,_af];
			_Pref1 setPos _epos;
			_oD = _obj distance _Pref1;
			_Pref1 setPos [0,0,0];
			if (_z < _clrH && _radcnt <= (_width / 2)) then {_los = false};
			if (format ["%1",_obj] != "<NULL-object>" && ! (_obj in _objA) && ! ((typeOf _obj) in _bad) && _los && _oD <= _fdis && _oD > _slop)
			then
				{
				_objA = _objA + [_obj];
				_typ = typeOf _obj;
				_Pref1 setPos (getPos _obj);
				_selfDis = _Pref1 distance _obj;
				_Pref1 setPos [getPos _obj select 0, getPos _obj select 1, 0];
				_oH = _selfDis * 2;
				_fH = (_Pref1 distance _obj) * 2;
				_Pref1 setPos [0,0,0];
				_oD = sqrt ((_x - (getPos _obj select 0))^2 + (_y - (getPos _obj select 1))^2);
				_wf = _selfDis / 2;
				_Dx = (getPos _obj select 0) - (_x);
				_Dy = (getPos _obj select 1) - (_y);
				if (_Dy == 0) then {_Dy = .0000001};
				_ang1 = _dir;
				_ang2 = _Dx atan2 _Dy;
				if (_ang2 < 0) then {_ang2 = _ang2 + 360};
				if ((_ang1 - _ang2) > 180) then {_ang2 = _ang2 + 360};
				if ((_ang2 - _ang1) > 180) then {_ang1 = _ang1 + 360};
				_doff = _ang2 - _ang1;
				_oRx = abs (_oD * sin _doff);
				if (_typ == "")
	 			   then
 				   {
				   if (abs (_selfDis - 3.75772) < .00001) then {_wf = 2.4};
				   if (_selfDis > 1.1 && _selfDis < 1.47) then {_wf = 3};
				   if (abs (_selfDis - 1.496) < .001 || abs (_selfDis - 2.08276) < .00001 || abs (_selfDis - .692902) < .000001) then {_wf = 2};
				   if (abs (_selfDis - 4.9532) < .0001) then {_wf = _oH / 2};
				   if (abs (_selfDis - 4.31347) < .00001) then {_wf = 4};
				   if (abs (_selfDis - 8.64056) < .00001) then {_wf = 10.5};
				   if (_selfDis > 5.5 && _selfDis < 6) then {_wf = _selfDis - 1.5};
				   if (_selfDis < .6 || abs (_selfDis - 1.2204) < .0001 || abs (_selfDis - .992462) < .000001) then {_wf = 0};
				   if (abs (_selfDis - 5.53625) < .00001) then {_wf = 1; _oH = _oH - 1.5};
				   if ((_selfDis > 6.8 && _selfDis < 6.9) || (_selfDis > 10.9 && _selfDis < 12.5) || _selfDis > 13) then {_wf = 25; _oH = _fH};
				   if (abs (_selfDis - 13.1648) < .0001) then {_wf = 6.5; _oH = 30};
				   };
				if (_typ != "")
				   then
				   {
				   _wf = 10;		
				   if (_typ == "ViewTower") then {_wf = 1};
				   if (_typ == "ED102_RadioTower") then {_wf = 1};
				   if (_typ == "Housekostel_trosky") then {_wf = 15};
				   if (_typ == "Housedulni_bs") then {_wf = 5.4};
				   if (_typ == "AAA548") then {_oH = _oH + 1.5};
				   if (_typ == "AAA549") then {_wf = 9.4};
				   if (_typ == "Housebudova1") then {_wf = 9.75};
				   if (_typ == "Housebudova2") then {_wf = 11; _oH = _oH + 2};
				   if (_typ == "Housebudova3") then {_wf = 5.8};
				   if (_typ == "Housebudova4") then {_wf = 9};
				   if (_typ == "Housebudova5") then {_wf = 6.6};
				   if (_typ == "FuelStation" || _typ == "Houseafhospoda_mesto") then {_wf = 11};
				   if (_typ == "Houserepair_center") then {_wf = 7.6};
				   if (_typ == "Housegaraz_bes_tanku") then {_wf = 11; _oH = _oH + 1.4};
				   if (_typ == "Housedum_mesto_in") then {_wf = 8; _oH = _oH - 3};
				   if (_typ == "Housedum_m2") then {_wf = 6; _oH = _oH + 1.7};
				   if (_typ == "Houseafbarabizna") then {_oH = _oH + 1.6};
				   if (_typ == "Houseadumo2") then {_wf = 8; _oH = _oH + 1};
				   if (_typ == "Housedumruina_mini") then {_wf = 9; _oH = _oH + 1.5};
				   if (_typ == "Housebouda1") then {_wf = 5; _oH = _oH + 1.9};
				   if (_typ == "Houseafdum_mesto21") then {_oH = _oH + 1.5};
				   };
				if (_oRx < ((_width / 2) + _wf) && _wf > 0)
				then
					{
					_oH = _oH + ((_wf * sin (Acos (_oRx / _wf))) * sin (abs _ang));
					_oRy = _oD * cos _doff;
					_Zref setPos [_x + (_oRy * sin _dir), _y + (_oRy * cos _dir)];
					_Pref2 setPos [_x + (_oRy * sin _dir), _y + (_oRy * cos _dir), 0];
					_oZ = _Zspos + ((_dis + _oRy) * sin _ang) - (_Zref distance _Pref2);
					_Zref setPos [0,0,0];
					_Pref2 setPos [0,0,0];
					if (_oH > (_oZ - _clrH)) then {_los = false};
					};	
				};
			_dang = _dang + _ainc;
			};
		_radcnt = _radcnt + _rinc;
		if (_radcnt > (_width / 2) && _radcnt < _rad) then {_radcnt = _rad};
		};
	_dis = _dis + _inc;
	};
deleteVehicle _Zref;
deleteVehicle _Pref1;
deleteVehicle _Pref2;
_los