/*---------------------------------------------------------------
LowAltDir function by Raptorsaurus

Given a starting position, a nominal direction, maximum permissible deviation
from the nominal direction, angular checking increments, path test point
increments, total range over which to check and a factor determining how much the
nominal direction should be favored, this function will return the direction that
will follow the lowest average altitude (finds the passes between mountains and
the valleys to follow).

Pass info to the function in the following format:

_lowdir = [_pos,_nomdir,_angdev,_anginc,_tpinc,_range,_opfav] call LowAltDir

Note:	The higher the value of _opfav the more likely the solution will be in the
	_nomdir.  To prevent strange path choices over water _opfav should be at
	least 3 to take into acount changing tide.

Initialize this function by putting this in your init.sqs:
LowAltDir  = preprocessFile "LowAltDir.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_pos","_nomdir","_angdev","_anginc","_tpinc","_range","_tang","_angcnt","_cnt"];
private ["_SLref","_Tref","_RecDir","_RecAlt","_x","_y","_Dalt","_lowdir","_hidir","_Dis"];
private ["_xTref","_yTref","_posTref","_Ang","_ASL","_Hdis","_Altsum","_AltAvg","_discnt"];

//get info

_pos = _this select 0;
_nomdir = _this select 1;
_angdev = _this select 2;
_anginc = _this select 3;
_tpinc = _this select 4;
_range = _this select 5;
_opfav = _this select 6;

_opfav = abs _opfav;

if (_nomdir <= 0) then {_nomdir = _nomdir + 360};
if (_nomdir > 360) then {_nomdir = _nomdir - 360};

// Determine the low and high compass directions to check

_lowdir = _nomdir - _angdev;
_hidir = _nomdir + _angdev;

//Initialize values

_RecDir = _nomdir;
_angcnt = _lowdir;
_RecAlt = 99999;
_array = [];

// Create Sea level reference and Test reference

_SLref = "emptydetector" camCreate [0,0,0];
_Tref = "logic" camCreate _pos;

// Starting with _lowdir count through the test directions

While {_angcnt < _hidir}
Do
{
   _tang = _angcnt;
   if (_tang <= 0) then {_tang = _tang + 360};
   if (_tang > 360) then {_tang = _tang - 360};

   // Measure the altitude at each test point increment and take average

   _Tref setPos _pos;
   _xTref = _pos select 0;
   _yTref = _pos select 1;
   _Altsum = 0;
   _discnt = _tpinc;
   _Altavg = 0;
   _cnt = 0;
   While {_discnt < _range}
   Do
   {   
      _cnt = _cnt + 1;	
      _Tref setPos [(_xTref + (_tpinc * Sin _tang)),(_yTref + (_tpinc * Cos _tang))];
      
      //Calculate ASL

      _PosTref = getPos _Tref;
      _xTref = _PosTref select 0;
      _yTref = _PosTref select 1;
      _SLref setPos [_xTref, _yTref];
      _ASL = _Tref distance _SLref;
    
      // Sum ASL values

      _Altsum = _Altsum + _ASL;
      _discnt = _discnt + _tpinc;
   };

   // Find the average altitude, if the average altitude is lower than that previously
   // recorded, then Record the direction and average altitude.

   _Altavg = _Altsum / _cnt;

   // Give _nomdir some favoritism (prevents strange veering over water or level ground)

   if (_angcnt > _nomdir - _anginc && _angcnt < _nomdir + _anginc) then {_Altavg = _Altavg - _opfav};
   if (_Altavg < _RecAlt) then 
   {
      _RecAlt = _Altavg;
      _RecDir = _tang;
   }; 
   
   // Increment to next check direction
   _angcnt = _angcnt + _anginc;
};

deleteVehicle _SLref;
deleteVehicle _Tref;

_RecDir