/*---------------------------------------------------------------
MovePosAcc function by Raptorsaurus
v. 1.01 (April, 18, 1010 - Fixed problem where gravitational acceleration of 0 would cause division by zero error)

Given an object, the initial speed, the trajectory direction and angle, trajectory
acceleration and gravitational acceleration and the elapsed time, this function will
place the object in a calculated postion along the theoretical trajectory.
Thus this function can be used in a loop to make it appear that an object has been
projected toward the target area. This is usefull for moving objects that do not
respond to the setVelocity command. This function also will return the calculated,
position, the updated theoretical trajectory speed, the updated trajectory angle
and the theoretical velocity (x,y,z).

The required info passed to this function is:

- Object name
- Simulated initial speed in m/s
- The direction of flight path
- The angle of flight path
- The trajectory acceleration (negative for effect of drag, positive for effect of thrust)
- Gravitational acceleration (-9.8 m/s for earth).
- Elapsed time in seconds

Example:

_MovePosAcc = preprocessFile "MovePosAcc.sqf"
_obj = "fire" createVehicle (getPos player)
_obj inflame true
_spd = 0
_dir = getDir player
_ang = 35
_acc = 30
_grav = -9.8
_rt = _time
_endtm = _time + 120
#loop
~.01
_etm = _time - _rt; _rt = _time
_data = [_obj, _spd, _dir, _ang, _acc, _grav, _etm] call _MovePosAcc
_spd = _data select 1
_ang = _data select 2
? _time > _endtm : _acc = -.1
? getPos _obj select 2 >= 0 : goto "loop"
deleteVehicle _obj

This will make the fireball1 look like a missile being fired at a 35 degree angle with
over 3 G thrust (30 m/s/s). The thrust will be for 2 minutes, then the fireball will
start to slow at -.1 m/s/s.

Initialize this function by putting this in your init.sqs:
MovePosAcc = preprocessFile "MovePosAcc.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_obj","_spd","_vspd","_tm","_Zref","_Pref","_Zobj","_Zpos","_hspd","_tacc","_gacc","_Zoff"];
private ["_x","_y","_z","_Hdis","_Vdis","_dir","_ang","_tspd","_tang","_hspdf","_vspdf","_opos"];

// get the parameters

_obj = _this select 0;
_spd = _this select 1;
_dir = _this select 2;
_ang = _this select 3;
_tacc = _this select 4;
_gacc = _this select 5;
_tm = _this select 6;

_Zref = "emptydetector" createVehicle [0,0,0];
_Pref = "logic" createVehicle [0,0,0];
_opos = getPos _obj;

// get object position relative to sea level and elevated roadway offset

_Pref setPos [_opos select 0, _opos select 1, 0];
_Zoff = _Pref distance _obj;
_Zref setPos [_opos select 0, _opos select 1];
_Zobj = (_Pref distance _Zref) + _Zoff;

// calculate the initial horizontal and vertical speed

_hspd = _spd * cos _ang;
_vspd = _spd * sin _ang;

// calculate the new trajectory speed (given _tacc and _tm)

_tspd = (_tacc * _tm) + _spd;

// calculate the new horizontal and vertical speed (given _gacc, _tm)

_hspdf = _tspd * cos _ang;
_vspdf = _tspd * sin _ang;
_vspdf = _vspdf + (_gacc * _tm);

// calculate the new trajectory angle and final speed at the new angle

if (_hspdf != 0)
   then
	{
	 _tang = atan (_vspdf / _hspdf);
	 _tspd = (_hspdf / Cos _tang); 
	}
   else
	{
	 if (_vspdf > 0) then {_tang = 90};
	 if (_vspdf < 0) then {_tang = -90};
	 if (_vspdf == 0) then {_tang = 0};
	 _tspd = (abs _vspdf); 
	};

// calulate the horizontal and vertical distance that will be covered in the given _tm at the average speed

_Hdis = (_hspdf + _hspd) / 2 * _tm;
_Vdis = (_vspdf + _vspd) / 2 * _tm;

// Calculate the x and y positions

_x = (_opos select 0) + (_Hdis * Sin _dir);
_y = (_opos select 1) + (_Hdis * Cos _dir);

// Calculate the z position compensated for sea level of ground at (_x, _y)

_z = _Zobj + _Vdis;
_Zref setPos [_x, _y];
_Pref setPos [_x, _y, 0];
_Zpos = _Pref distance _Zref;
_z = _z - _Zpos;

// Calculate the x and y velocity

_xvel = _hspdf * Sin _dir;
_yvel = _hspdf * Cos _dir;

deleteVehicle _Zref;
deleteVehicle _Pref;

_obj setPos [_x, _y, _z];
[[_x, _y, _z],_tspd,_tang,[_xvel,_yvel,_vspdf]]