/*---------------------------------------------------------------
MovePosGrav function by Raptorsaurus
v. 1.1 (June 11, 2008 - Function now returns theoretical velocity)

Given an object, a final position, the initial vertical speed, the gravitational
acceleration and the elapsed time, this function will place the object in a
calculated postion on a parapolic path toward the final position. The required
theoretical horizontal speed will be calculated by the function.
Thus this function can be used in a loop to make it appear that an object has been
projected toward the target area.  This is usefull for moving objects that do not
respond to the setVelocity command.  This function also will return the calculated,
position, the theoretical horizontal speed, the theoretical vertical speed and the
theoretical velocity [x,y,z].

The required info passed to this function is:

- Object name
- Final destination position [x, y, z]
- Simulated initial vertical speed in m/s (use negative for downward and positive for upward)
- Vertical acceleration (negative for gravity, positive for upward thrust/lift)
- Elapsed time in seconds

Example:

_MovePosGrav = preprocessFile "MovePosGrav.sqf"
_obj = fireball1
_pos = (getPos house)
_vspd = 0
_acc = -9.8
_rt = _time
#loop
~.01
_etm = _time - _rt
_data = [_obj, _pos, _vspd, _acc, _etm] call _MovePosGrav
_vspd = _data select 2
_rt = _time
? (_obj distance house > 1) : goto "loop"

This will make the fireball1 move toward the house in a seemingly parapolic
trajectory. Once the fireball hits the house, the loop will end.

Initialize this function by putting this in your init.sqs:
MovePosGrav = preprocessFile "MovePosGrav.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_obj", "_pos", "_vspd", "_tm", "_Zref1", "_Pref", "_Zobj", "_Zpos", "_opos", "_hspd", "_ft1", "_ftime"];
private [ "_Hdis", "_x", "_y", "_z", "_dis", "_Hdis", "_Vdis", "_dir", "_Dx", "_Dy", "_Zref2", "_acc", "_ft2"];
private [ "_xvel","_yvel"];

// get the parameters

_obj = _this select 0;
_pos = _this select 1;
_vspd = _this select 2;
_acc = _this select 3;
_tm = _this select 4;

_Zref1 = "emptydetector" createVehicle [0,0,0];
_Zref2 = "emptydetector" createVehicle [0,0,0];
_Pref = "logic" createVehicle [0,0,0];
_opos = getPos _obj;

// get the verticle and horizontal distance to the final position

_Pref setPos _opos;
_Zref1 setPos [_opos select 0, _opos select 1];
_Zref2 setPos [_pos select 0, _pos select 1];
_Zobj = _Pref distance _Zref1;
_Pref setPos _pos;
_Zpos = _Pref distance _Zref2;
_Hdis = _Zref1 distance _Zref2;
_Vdis = _Zpos - _Zobj;

// calculate the total time to reach the target position

_ft1 = (-_vspd + sqrt (_vspd^2 - (2 * _acc * -_Vdis))) / _acc;
_ft2 = (-_vspd - sqrt (_vspd^2 - (2 * _acc * -_Vdis))) / _acc;

_ftime = _ft2;
if (_vspd * _acc < 0) then {if (_ft1 > _ft2) then {_ftime = _ft1} else {_ftime = _ft2}};
if (_ft1 < 0) then {_ftime = _ft2};
if (_ft2 < 0) then {_ftime = _ft1};

// calculate the horizontal speed needed to go _Hdis in _ftime (total trajectory time).

_hspd = _Hdis / _ftime;

// calculate the new vertical speed (given _acc and _tm)

_vspd = (_acc * _tm) + _vspd;

// calulate the horizontal distance that will be covered in the given _tm at _hspd

_Hdis = _hspd * _tm;

// calculate the vertical distance that will be covered in the given _tm at _vspd

_Vdis = _vspd * _tm;

// get direction to position

_Dx = (_pos select 0) - (_opos select 0);
_Dy = (_pos select 1) - (_opos select 1);
if (_Dy == 0) then {_Dy = .0000001};
_dir = _Dx atan2 _Dy;
if (_dir < 0) then {_dir = _dir + 360};

// Calculate the x and y positions

_x = (_opos select 0) + (_Hdis * Sin _dir);
_y = (_opos select 1) + (_Hdis * Cos _dir);

// Calculate the z position compensated for sea level of ground at (_x, _y)

_z = _Zobj + _Vdis;
_Zref1 setPos [_x, _y];
_Pref setPos [_x, _y, 0];
_Zpos = _Pref distance _Zref1;
_z = _z - _Zpos;

// Calculate the x and y velocity

_xvel = _hspd * Sin _dir;
_yvel = _hspd * Cos _dir;

deleteVehicle _Zref1;
deleteVehicle _Zref2;
deleteVehicle _Pref;

_obj setPos [_x, _y, _z];
[[_x, _y, _z],_hspd, _vspd, [_xvel,_yvel,_vspd]]