/*---------------------------------------------------------------
ObjDirAngtoObj function by Raptorsaurus

Returns the relative direction and angle from one object to another

Pass the reference object and target object and optional offsets
as follows:

_Dirdata = [_refobj,_targobj] call ObjDirAngtoObj

or with optional offset

_Dirdata = [_refobj,_targobj, [[_refx, _refy, _refZ],[_targx, _targy, _targZ]] call ObjDirAngtoObj

_Dirdata contains two values:

_bearing = _Dirdata select 0
_elev = _Dirdata select 1

_bearing is the relative direction to the target object (relative to the reference object's facing direction).
_elev is the angle to the target object. If optional offesets are used the starting and ending points will
	be altered, and therefore the _bearing and _elev will be affected. The option offset are relative to
	the _refobj and _targobj. So if you want to find the angle and direction from the point 3 meters
	behind the _refobj to the point 2 meters left and 1.5 meters above the _targobj you put this:

_Dirdata = [_refobj,_targobj, [[0, -3, 0],[-3, 0, 1.5]] call ObjDirAngtoObj

Initialize this function by putting this in your init.sqs:
ObjDirAngtoObj = preprocessFile "ObjDirAngtoObj.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_Robj","_Tobj", "_dirRobj", "_bearTobj","_Reldir","_Roff", "_Toff"];
private ["_Dis","_Zref","_Ang","_RobjASL","_TobjASL","_Dalt", "_deltaY", "_deltaX"];
private ["_X","_y","_z","_Pref", "_Oref", "_Tref", "_dir", "_cnt", "_pos", "_obj", "_SLref"];
private ["_Xoff","_Yoff","_Zoff","_leadX", "_leadY", "_leewX", "_leewY", "_SLpos"];

_Robj = _this select 0;
_Tobj = _this select 1;
_Roff = [0,0,0];
_Toff = [0,0,0];

if (count _this > 2) then {_Roff = (_this select 2) select 0; _Toff = (_this select 2) select 1};

// Create detectors

_Zref = "emptydetector" createVehicle [0,0,0];
_Oref = "logic" createVehicle [0,0,0];
_Tref = "logic" createVehicle [0,0,0];

// Find offset postitions relative to _Robj and _Tobj

_cnt = 0;

While {_cnt <= 1} Do
	{
	_obj = _this select _cnt;
	if (_obj == _Robj) then
		{
		_Xoff = _Roff select 0;
		_Yoff = _Roff select 1;
		_Zoff = _Roff select 2;
		_Pref = _Oref;
		}
		else
		{
		_Xoff = _Toff select 0;
		_Yoff = _Toff select 1;
		_Zoff = _Toff select 2;
		_Pref = _Tref;
		};
	_dir = getDir (vehicle _obj);
	_pos = getPos _obj;

	_x = _pos select 0;
	_y = _pos select 1;

	// Setup position relative to _obj

	_leadX = (_Yoff * Sin _dir);
	_leadY = (_Yoff * Cos _dir);
	_leewX = (_Xoff * Sin (_dir + 90));
	_leewY = (_Xoff * Cos (_dir + 90));

	// Correct for sea level at offset position and calculate _z

	_Zref setPos [getPos _obj select 0, getPos _obj select 1];
	_SLref = _obj distance _Zref;
	_Zref setPos [ (_leadX + _leewX + _x) , (_leadY + _leewY + _y) ];
	_Pref setPos [ (_leadX + _leewX + _x) , (_leadY + _leewY + _y), 0 ];
	_SLpos = _Pref distance _Zref;
	_z = (_SLref + _Zoff) - _SLpos;
	_Pref setPos [ (_leadX + _leewX + _x) , (_leadY + _leewY + _y) ,_z];
	_Pref setDir _dir;
	_cnt = _cnt + 1;
	};

_dirRobj = getDir _Oref;

// calculate bearing to target

_deltaX = (getPos _Tref select 0) - (getPos _Oref select 0);
_deltaY = (getPos _Tref select 1) - (getPos _Oref select 1);

if (_deltaY == 0) then {_deltaY = .0000001};

_bearTobj = _deltaX atan2 _deltaY;

// set bearing to value that can be used to get relative direction

if (_bearTobj < 0) then {_bearTobj = _bearTobj + 360};

// get relative direction (from _Oref to _Tref)

_Reldir = _bearTobj - _dirRobj;

// make sure it is in 360 degree format

if (_Reldir <= 0) then {_Reldir = _Reldir + 360};
if (_Reldir > 360) then {_Reldir = _Reldir - 360};

// get distance and ASL altitude info

//Calculate ASL of _Oref

_Zref setPos [getPos _Oref select 0, getPos _Oref select 1];
_RobjASL = _Oref distance _Zref;

//Calculate ASL of _Tobj

_Zref setPos [getPos _Tref select 0, getPos _Tref select 1];
_TobjASL = _Tref distance _Zref;

// calculate delta altitude

_Dalt = _TobjASL - _RobjASL;

// get distance from _Oref and _Tref

_Dis = _Oref Distance _Tref;

// get relative angle to target

if (_Dis == 0) then {_Dis = .000001};
_Ang = Asin (_Dalt/_Dis);

//Delete the detectors and return direction and angle

deleteVehicle _Zref;
deleteVehicle _Oref;
deleteVehicle _Tref;

[_Reldir,_Ang]