/*---------------------------------------------------------------
ObjsOfTypeInRange function by Raptorsaurus

Returns an array of all the objects of given types within a circular or spherical range of a reference object and
which of those objects is the closest. General type names ("man", "car", "air") will work as well as more specific
type names ("soldierWB", "soldierWLAW", "jeepMG").

The required info passed to this function is:

- Reference Object
- Array of object types to search for (e.g. ["man", "tank", "car"])
- Radius (the radius of the circle or sphere)
- increment (the minimum distance increment between positions to check)
- "3D" or "2D" (3D for sphere, 2d for circle)
- [zLow, zHigh] (the z value range relative to the OFP terrain level), 2 dim. array with first element low limit and second element high limit)

Examples:

_objinfo = [ player, ["tank", "car"], 30, 2, "3d", [-30, 30]] call ObjsOfTypeInRange
_objects = _objinfo select 0
_closest = _objinfo select 1

_objects is an array of all the "tank" and "car" objects within a sphere of 30 m radius around the player with the test positions being 2 m apart.
Since the z limit is [-30, 30] the sphere of positions will be uneffected by the distance above/below ground.
_closest is that object of the given types, that is closest to the player.

_objinfo = [ player, ["tank", "car"], 30, 5, "2d", [0, 0]] call ObjsOfTypeInRange
_objects = _objinfo select 0
_closest = _objinfo select 1

_objects is an array of all the "tank" and "car" objects within a circle of 30 m radius around the player with the test positions being 5 m apart.
Since z limit is [0,0] no postions will exist below or above ground level, the check area will therefore follow the terrain contour.
_closest is that object of the given types, that is closest to the player.

Initialize this function by putting this in your init.sqs:
ObjsOfTypeInRange = preprocessFile "ObjsOfTypeInRange.sqf"
---------------------------------------------------------------*/
private ["_refobj","_rad","_vinc","_SLref","_GLref","_Gsla","_ASLcomp","_ASL","_objdir","_inc","_zLow","_zHi","_obj","_tcnt","_rdis"];
private ["_vang","_hang","_radcnt","_X","_Y","_Z","_H","_rX","_rY","_rZ","_objpos","_dim3","_objA","_hinc","_tmp","_types","_typ","_nobj"];
_refobj = _this select 0;
_types = [];
_types = _this select 1;
_rad = _this select 2;
_inc = _this select 3;
_dim3 = false;
if (_this select 4 == "3D" || _this select 4 == "3d") then {_dim3 = true};
_zLow = (_this select 5) select 0;
_zHi = (_this select 5) select 1;
if (_zLow > _zHigh) then {_tmp = _zLow; _zLow = _zHi; _zHi = _tmp};
_objpos = getPos _refobj;
_objdir = getDir _refobj;
_SLref = "emptydetector" createVehicle [0,0,0];
_GLref = "logic" createVehicle [0,0,0];
_objA = [_refobj];
_SLref setPos [_objpos select 0, _objpos select 1];
_ASL = _refobj distance _SLref;
_rdis = 999999999;
_nobj = "";

_radcnt = _inc;
While {_radcnt <= _rad}
Do
{
	_vinc = 360 / ( (360 / Asin (_inc/_radcnt)) - ((360 / Asin (_inc/_radcnt)) % 1) );
	_vang = 0;
	if (_dim3) then {_vang = -90};
	// vertical angle scan loop
	While {_vang <= 90} 
	Do
	{
		_rZ = _radcnt * Sin _vang;
		_H = _radcnt * Cos _vang;
		_hinc = 360 / ( (360 / Asin (_inc/_H)) - ((360 / Asin (_inc/_H)) % 1) );
		if (format ["%1", _hinc] == "-1.#IND" && format ["%1", (_inc/_H)] == "1") then {_hinc = 90};
		_hang = 0;
		// horzontal angel scan loop
		While {_hang < 360}
		Do
		{
			_rX = _H * Sin _hang;
			_rY = _H * Cos _hang;
			_X = (_objpos select 0) + (_rY * Sin _objdir) + (_rX * Sin (_objdir + 90));
			_Y = (_objpos select 1) + (_rY * Cos _objdir) + (_rX * Cos (_objdir + 90));
			_GLref setPos [_X, _Y, 0];
			_SLref setPos [_X, _Y];
			_Gsla = _GLref distance _SLref;
			_ASLcomp = _ASL - _Gsla;
			_Z = _ASLcomp + _rZ;
			_GLref setPos [0,0,0];
			_SLref setPos [0,0];
			if (_Z < _zLow) then {_Z = _zLow};
			if (_Z > _zHi) then {_Z = _zHi};
			_obj = nearestObject [_X, _Y, _Z];
			// _types check loop
			_tcnt = 0;
			While {_tcnt <= (count _types) - 1 && ! (_obj in _objA) && (_refobj distance _obj) <= _rad && ! isNull _obj}
			Do
			{
				_typ = _types select _tcnt;
				if (_typ countType [_obj] == 1 || typeOf _obj in _types)
				then
				{
					_objA = _objA + [_obj];
					if (_obj distance _refobj < _rdis) then {_nobj = _obj; _rdis = _obj distance _refobj};
				};
				_tcnt = _tcnt + 1;
			};
			_hang = _hang + _hinc;
		};
		if (_dim3) then {_vang = _vang + _vinc} else {_vang = 95};
	};
	_radcnt = _radcnt + _inc;
};

deleteVehicle _GLref;
deleteVehicle _SLref;

[(_objA - [_refobj]), _nobj]