/*---------------------------------------------------------------
PosArray function by Raptorsaurus

Returns an array of positions forming either a sphere or circle around on object.
Positions that have a z value less than zero will be changed to zero z positions
so that the postions will follow the terrain contour.

The required info passed to this function is:

- Reference Object
- Radius (the radius of the circle or sphere)
- increment (the minimum distance increment between positions)
- "3D" or "2D" (3D for sphere, 2d for circle)
- [zLow, zHigh] (the z value range relative to the OFP terrain level), 2 dim array with first element low limit and second element high limit)
- <Optional> minimum starting radius. If you want to have a "hole" at the center of the circular/spherical array pattern, then this will be the radius of that hole. Leave this parameter out, or make it 0 (zero) for no "hole".

Examples:

_posA = [ player, 30, 2, "3d", [-30, 30] call PosArray
This will yeild an array of all the positions within a sphere of 30 m radius around the player with the positions being 2 m apart.
Since the z limit is [-30, 30] the sphere of positions will be uneffected by the distance above/below ground.

_posA = [ player, 30, 5, "2d", [0, 0]] call PosArray
This will yeild an array of all the positions within a circle of 30 m radius around the player with the positions being 2 m apart.
Since zRange is [0,0] no postions will exist below or above ground level, the circle will therefore follow the terrain contour.

_posA = [ player, 30, 5, "2d", [0, 0], 10] call PosArray
This will return a similar array to the above example, but there will be a 10m radius "hole" at the center of the circular array. 

Initialize this function by putting this in your init.sqs:
PosArray = preprocessFile "PosArray.sqf"
---------------------------------------------------------------*/
private ["_refobj", "_rad", "_vinc", "_SLref", "_GLref", "_Gsla", "_ASLcomp", "_ASL", "_objdir", "_inc", "_zLow", "_zHi", "_mrad"];
private ["_vang","_hang","_radcnt","_X","_Y","_Z", "_H", "_rX", "_rY", "_rZ", "_objpos", "_dim3", "_posA", "_hinc", "_tmp"];
_refobj = _this select 0;
_rad = abs (_this select 1);
_inc = abs (_this select 2);
if (_inc < _rad) then {_inc = _rad};
_dim3 = false;
if (_this select 3 == "3D" || _this select 3 == "3d") then {_dim3 = true};
_zLow = (_this select 4) select 0;
_zHi = (_this select 4) select 1;
if (_zLow > _zHigh) then {_tmp = _zLow; _zLow = _zHi; _zHi = _tmp};
_objpos = getPos _refobj;
_objdir = getDir _refobj;
_SLref = "emptydetector" createVehicle [0,0,0];
_GLref = "logic" createVehicle [0,0,0];
_SLref setPos [_objpos select 0, _objpos select 1];
_ASL = _refobj distance _SLref;
_mrad = 0;
if (count _this > 5) then {_mrad = abs (_this select 5)};
if (_mrad == 0) then {_posA = [_objpos]} else {_posA = []};
if (_mrad >= _rad) then {hint "ERROR calling PosArray.sqf: Minimun radius must be less than radius (parameter 5 must be smaller than parameter 2)."};
if (_inc < (_rad - _mrad)) then {_inc = _rad - _mrad};
_radcnt = _inc + _mrad;
While {_radcnt <= _rad}
	Do
	{
	_vinc = 360 / ((360 / Asin (_inc/_radcnt)) - ((360 / Asin (_inc/_radcnt)) % 1));
	_vang = 0;
	if (_dim3) then {_vang = -90};
	// vertical angle scan loop
	While {_vang <= 90} 
		Do
		{
		_rZ = _radcnt * Sin _vang;
		_H = _radcnt * Cos _vang;
		_hinc = 360 / ((360 / Asin (_inc/_H)) - ((360 / Asin (_inc/_H)) % 1));
		if (format ["%1", _hinc] == "-1.#IND" && format ["%1", (_inc/_H)] == "1") then {_hinc = 90};
		_hang = 0;
		While {_hang < 360}
			Do
			{
			_rX = _H * Sin _hang;
			_rY = _H * Cos _hang;
			_X = (_objpos select 0) + (_rY * Sin _objdir) + (_rX * Sin (_objdir + 90));
			_Y = (_objpos select 1) + (_rY * Cos _objdir) + (_rX * Cos (_objdir + 90));
			_GLref setPos [_X, _Y, 0];
			_SLref setPos [_X, _Y];
			_Gsla = _GLref distance _SLref;
			_ASLcomp = _ASL - _Gsla;
			_Z = _ASLcomp + _rZ;
			if (_Z < _zLow) then {_Z = _zLow};
			if (_Z > _zHi) then {_Z = _zHi};
			_posA = _posA + [[_X, _Y, _Z]];
			_hang = _hang + _hinc;
			};
		if (_dim3) then {_vang = _vang + _vinc} else {_vang = 95};
		};
	_radcnt = _radcnt + _inc;
	};

deleteVehicle _GLref;
deleteVehicle _SLref;
_posA