/*---------------------------------------------------------------
PosArrayRec function by Raptorsaurus
v. 1.0

Creates a rectangular array of positions along a path between two points. Optionally,
the array will leave a user defined gap along the defining path. All positions will be
at ground level.

The required info passed to this function is:

- Starting postion [x, y] or [x, y, z].
- Final destination position [x, y] or [x, y, z].
- Width of position array.
- Increment between positions.
- Optional gap width along path center line.

Example 1:

_spos = [getPos shooter select 0, getPos shooter select 1]
_epos = [getPos target select 0, getPos target select 1]
_width = 25
_inc = 5

_positions = [_spos, _epos, _width, _inc] call PosArrayRec

This returns a rectangular array of positions between the shooter and the target 25 meters
wide with 5 meters spacing between the positions.

Example 2:

_spos = [getPos shooter select 0, getPos shooter select 1]
_epos = [getPos target select 0, getPos target select 1]
_width = 25
_inc = 5
_gap = 10

_positions = [_spos, _epos, _width, _inc, _gap] call PosArrayRec

This returns a rectangular array of positions between the shooter and the target 25 meters
wide with 5 meters spacing between the positions. However, the array will not include any
positions along a 10 meter wide gap down the center of the path between the shooter and
the target.

Initialize this function by putting this in your init.sqs:
PosArrayRec = preprocessFile "PosArrayRec.sqf"

Or as a local variable in a script:
_PosArrayRec = preprocessFile "PosArrayRec.sqf"

---------------------------------------------------------------*/

// declare private variables
private ["_spos","_epos","_inc","_pinc","_winc","_posarray","_wdir"];
private ["_x","_y","_dis","_fdis","_dir","_gap","_wx","_wy","_wdis"];


// get the parameters

_spos = _this select 0;
_epos = _this select 1;
_width = _this select 2;
_inc = _this select 3;
_gap = 0;
if (count _this > 4) then {_gap = _this select 4};

_posarray = [];

// get the distance to the final position

_fdis = sqrt (((_spos select 0) - (_epos select 0))^2 + ((_spos select 1) - (_epos select 1))^2);

// get direction to position

_wx = (_epos select 0) - (_spos select 0);
_wy = (_epos select 1) - (_spos select 1);
if (_wy == 0) then {_wy = .0000001};
_dir = _wx atan2 _wy;
if (_dir < 0) then {_dir = _dir + 360};

// optimize check increments

_pinc = _fdis / ((_fdis/_inc) - ((_fdis/_inc) % 1));
_winc = (_width - _gap) / (((_width - _gap)/_inc) - (((_width - _gap)/_inc) % 1));

// add positions to _posarray

_dis = 0;
While {_dis <= _fdis}
Do
	{
	_x = (_spos select 0) + (_dis * sin _dir);
	_y = (_spos select 1) + (_dis * cos _dir);
	if (_gap == 0) then {_posarray = _posarray + [[_x,_y,0]]};
	_wdis = (_gap / 2) + _winc;
	While {_wdis <= _width / 2}
	Do
		{
		_wdir = _dir + 90;
		_wx = _x + (_wdis * sin _wdir);
		_wy = _y + (_wdis * cos _wdir);
		_posarray = _posarray + [[_wx,_wy,0]];
		_wdir = _dir - 90;
		_wx = _x + (_wdis * sin _wdir);
		_wy = _y + (_wdis * cos _wdir);
		_posarray = _posarray + [[_wx,_wy,0]];
		_wdis = _wdis + _winc;
		};
	_dis = _dis + _pinc;
	};
_posarray