/*---------------------------------------------------------------
RandPos function by Raptorsaurus

Returns a random position given a reference postition, reference
direction, minimum random distance, maximum random distance,
minimum random direction, maximum random direction, minimum
random elevational angle, maximum random elevational angle and
finally random value precision.

The required info passed to this function is:

- Position (either [x,y] or [x,y,z] format)
- Reference direction (0 to 360, compass format)
- Distance range (min & max)
- Desired direction range -180 to 180 or 0 to 360, min. & max.)
- Desired angle (90 [straight up] to -90 [straight down], min. & max.)
- Desired precision for random value

Example:

_pos =  getPos _plane1
_dir = getDir _plane1
_dismin = 50
_dismax = 150
_dirmin = -45
_dirmax = 45
_angmin = -30
_angmax = 30
_precis = 1

_RndPos = [_pos,_dir,_dismin,_dismax,_dirmin,_dirmax,_angmin,_angmax,_precis] call RandPos

_object2 setPos _RndPos

This will set the _object2 at a position in the range of 50 to 150m 
away from _plane1, in a direction relative to _plane1's direcetion
from 45 degrees to the left (-45) to 45 deg. to the right (45) and 
at an elevational angle of 30 degrees above _plane1 to 30 dgrees below
_plane1.  All values will have a precision to the nearest integer (1).

Initialize this function by putting this in your init.sqs:
RandPos = preprocessFile "RandPos.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_pos","_dirmin","_dirmax","_angmin","_angmax","_H","_precis"];
private ["_dismin","_dismax","_x","_y","_z","_dis","_dir","_ang","_refdir"];
private ["_ang1","_ang2"];

// get the parameters

_pos = _this select 0;
_refdir = _this select 1;
_dismin = _this select 2;
_dismax = _this select 3;
_dirmin = _this select 4;
_dirmax = _this select 5;
_angmin = _this select 6;
_angmax = _this select 7;
_precis = _this select 8;

// if _pos is only two elements, then add z element

if (count _pos < 3) then {_pos = _pos + [0]};

// make sure _dirmin, _dirmax are valid values (-180 to 180)

if (_dirmin > 180 && _dirmin <= 360) then {_dirmin = -(360 - _dirmin)};
if (_dirmin < -180) then {_dirmin = -180};
if (_dirmax > 180 && _dirmax <= 360) then {_dirmax = -(360 - _dirmax)};
if (_dirmax < -180) then {_dirmax = -180};
if (abs _dirmin == 180 && abs _dirmax == 180) then {_dirmin = -180 ; _dirmax = 180};
if (_dirmin == 0 && _dirmax == 0) then {_dirmin = -180 ; _dirmax = 180};

// make sure _angmin and _angmax are valid values

if (_angmin > 90) then {_angmin = 90};
if (_angmin < -90) then {_angmin = -90};
if (_angmax > 90) then {_angmax = 90};
if (_angmax < -90) then {_angmax = -90};

// Calculate Random values and round them to requested precision

_dis = (random (_dismax - _dismin)) + _dismin;
_dis = (_dis + (_precis/2)) - ((_dis + (_precis/2)) % _precis);
if (_dis < _dismin) then {_dis = _dismin};
if (_dis > _dismax) then {_dis = _dismax};
_dir = (random (_dirmax - _dirmin)) + _dirmin;
_dir = (_dir + (_precis/2)) - ((_dir + (_precis/2)) % _precis);
if (_dir < _dirmin) then {_dir = _dirmin};
if (_dir > _dirmax) then {_dir = _dirmax};
_ang = (random (_angmax - _angmin)) + _angmin;
_ang = (_ang + (_precis/2)) - ((_ang + (_precis/2)) % _precis);
if (_ang < _angmin) then {_ang = _angmin};
if (_ang > _angmax) then {_ang = _angmax};

// make sure _dir is in 0 to 360 format

if (_dir > 360) then {_dir = _dir - 360};
if (_dir < 0) then {_dir = _dir + 360};
if (_dir == 360) then {_dir = 0};

//Find relative directional offset

_ang1 = _refdir;
_ang2 = _dir;

if ((_ang1 - _ang2) > 180) then {_ang2 = _ang2 + 360};
if ((_ang2 - _ang1) > 180) then {_ang1 = _ang1 + 360};
_dir = (_ang1 + _ang2);
if (_dir > 360) then {_dir = _dir - 360};

// calculate vertical position component

_z = (_pos select 2) + _dis * Sin _ang;

// calculate horizontal distance component

_H = _dis * Cos _ang;

// calculate position components in horizontal plane

_x = (_pos select 0) + _H * Sin _dir;
_y = (_pos select 1) + _H * Cos _dir;

// new position array

[_x,_y,_z]