/*---------------------------------------------------------------
RelMovePos function by Raptorsaurus
v. 1.0

Given a reference object, an initial postion offset relative to that object, a final position
offset relative to that object, the simulated speed and the elapsed time, this function
returns the relative position that would be reached in the elapsed time, by a particle traveling
at that speed relative to the reference object. 

The required info passed to this function is:

- Reference Object name
- Starting relative offset position
- Final relative offset position
- Simulated speed in m/s
- Elapsed time in seconds

Example:

_refobj = myplane
_obj = fireball
_startpos = [-10, -25, 15
_endpos = [10, 50, 5]
_spd = 20
_rt = _time
_tmlim = _time + 5
#loop
_etm = _time - _rt; _rt = _time
_relpos = [_refobj, _startpos, _endpos, _spd, _etm] call RelMovePos
[_refobj, _obj, (_relpos select 1), (_relpos select 0), (_relpos select 2)] call SetObjPosObj
~.01
? _time < _tmlim : goto "loop"

This will make the fireball move from 10 m to the left of the plane, -25 m behind the plane and
15 m above the plane to 10 m to the right of the plane, 50 m in front of the plane and 5 m above
the plane.  The fireball will appear to be moving at 20 m/s relative to the plane.

Initialize this function by putting this in your init.sqs:
RelMovePos = preprocessFile "RelMovePos.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_refobj", "_posS", "_posF", "_spd", "_tm"];
private [ "_Hdis", "_x", "_y", "_z", "_dis", "_Fdis", "_ang", "_dir", "_Dx", "_Dy"];

// get the parameters

_refobj = _this select 0;
_posS = _this select 1;
_posF = _this select 2;
_spd = _this select 3;
_tm = _this select 4;

// get the distance to the final position

_Dx = (_posF select 0) - (_posS select 0);
_Dy = (_posF select 1) - (_posS select 1);

_Hdis = sqrt (_Dx ^2 + _Dy ^2);
_Vdis = (_posF select 2) - (_posS select 2);
_Fdis = sqrt (_Hdis ^2 + _Vdis ^2);

// calculate elevation angle to _posF

If (_Hdis == 0) then
{
  If (_Vdis > 0) then {_ang = 90};
  If (_Vdis < 0) then {_ang = -90};
  If (_Vdis == 0) then {_ang = 0};
}
else
{ _ang = Atan (_Vdis/_Hdis)};

// get direction to _posF

if (_Dy == 0) then {_Dy = .0000001};
_dir = _Dx atan2 _Dy;
if (_dir < 0) then {_dir = _dir + 360};

// calculate the movement distance over elapsed time (_tm)

_dis = _spd * _tm;
if (_dis > _Fdis) then {_dis = _Fdis};

// calculate the horizontal & vertical distances

_Hdis = _dis * cos _ang;
_z = _dis * sin _ang;

// Calculate the updated position

_x = (_posS select 0) + (_Hdis * Sin _dir);
_y = (_posS select 1) + (_Hdis * Cos _dir);
_z = _z + (_posS select 2);

[_x, _y, _z]