/*---------------------------------------------------------------
SetSpd function by Raptorsaurus

Returns the Velocity vectors required to alter a moving object's speed while
maintaining its current compass heading and angle of climb or dive.

The required info passed to this function is:

- Object name
- desired speed in k/h (kilometers per hour)
- desired direction type (this is a boolean, true to use the objects curent movement
  direction to get its compass heading, or false to use the getDir value for compass
  heading).

Example Script to slow a plane called myplane down to 150 k/h, then maintain it
at that speed for 120 seconds:

_obj = myplane
_spd = speed _obj
_dirtype = true

_finalspd = 150

#slowloop
_spd = _spd - 1
_Vel = [_obj,_spd,_dirtype] call SetSpd
_obj setVelocity _Vel
~.01
?(_spd > _finalspd): goto "slowloop"
_reftime = time
#holdloop
_Vel = [_obj,_spd,_dirtype] call SetSpd
_obj setVelocity _Vel
~.1
_chktime = time
?(_chktime - _reftime < 120) : goto "holdloop"
exit

Take care when seting aircraft to speeds that are too slow, they may stall and
crash.  Also, if you force some aircaft (especially helocopters) to go too fast
they become uncontrolable.

Initialize this function by putting this in your init.sqs:
SetSpd = preprocessFile "SetSpd.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_obj","_dir","_Hvel","_CurSpd","_Xvel","_Yvel","_Zvel"];
private ["_x","_y","_z","_tang","_vel","_Spd","_dirtype"];

// get the desired angle and current velocity (the velocity is a more reliable
// way to find speed and direction than the "Speed" and "getDir" commands)

_obj = _this select 0;
_Spd = _this select 1;
_dirtype = _this select 2;

// convert _Spd to m/s

_Spd = _spd * 1000 / 3600;

_vel = Velocity _obj;

// get the rectangular velocity vector components

_x = _vel select 0;
_y = _vel select 1;
_z = _vel select 2;

// find current speed and angle of climb/dive from velocity vectors

_Hvel = sqrt(_x^2 + _y^2);

If (_Hvel == 0) then
{
  If (_z > 0) then {_tang = 90};
  If (_z < 0) then {_tang = -90};
  If (_z == 0) then {_tang = 0};
};
If (_Hvel != 0)
then
{
  _tang = Atan (_z/_Hvel);
  _CurSpd = _Hvel / Cos _tang;
}
Else
{
  _CurSpd = abs _z;
};

// find compass direction according to movement direction unless
// _dirtype is false or object is not moving (then use "getdir".

if (_CurSpd > 0 && _x != 0 && _y != 0 && _dirtype) then {_dir = _x atan2 _y} else {_dir = getDir _obj};
if (_dir < 0) then { _dir = _dir + 360 };

// calculate the projected horizontal velocity component at desired _spd

_Hvel = _Spd * Cos _tang;

// calculate projected velocity components in horizontal plane

_Xvel = _Hvel * Sin _dir;
_Yvel = _Hvel * Cos _dir;

// calculate vertical velocity component

_Zvel = _Spd * Sin _tang;

// new velocity array to set object at desired speed

[_Xvel,_Yvel,_Zvel]
