/*---------------------------------------------------------------
TypesInTraj function by Raptorsaurus

Returns arrays of all targest of given types along the trajectory of a projectile, the target that is closest
to the trajectory path and the perpendicular distance from the closest target to the trajectory path.

The required info passed to this function is:

- The projectile
- An array of potential targets types (ie. ["LandVehicle", "Air", "Ship", "Man", "Building", "Fortress"])
- distance (the distance ahead of the projectile to check)
- increment (the minimum distance increments to check between the projectile and the furthest distance)
- max distance (targets that are more than the max perpedicular distance from the projectile path will be ignored)
- (OPTIONAL) excluded types array (array of types of targets to not include - like if you want skip invisible targets for instance)

Example:

_targs = [_missile, ["man", "air"], 2000, 10, 5, ["soldierWB", "soldierGB", "civilian"]] call TypesInTraj

Targets of the type "soldierWB", "soldierGB" and "civilian" will be excluded as potential targets.

_targs is a 3 element array.  The first element is an array of all the ["man", "air"] targets within 5 meters of the
trajectory path. The second element is the single "man" or "air" target that is closest to the trajectory path. The third
element is the perpendicular distance from that single target to the trajectory path. 

_allTargs = _targs select 0:
This is an array of all the targets matching the designated types nearest to the path of the missile up to 2000 meters away
(excluding those specified in the exclude array ["soldierWB", "soldierGB", "civilian"]). The check increments will be every
10 meters.

_BestTarg = _targs select 1:
This is the target that is nearest to the missiles trajectory (the target from the types array that the missile is most
likey to hit.)

_BestDis = _targs select 2:
This is the perpendicular distance from _BestTarg to the trajectory path. 

Initialize this function by putting this in your init.sqs:
TypesInTraj = preprocessFile "TypesInTraj.sqf"
---------------------------------------------------------------*/
private ["_proj", "_dis", "_dir", "_inc", "_obj", "_Pvel", "_XvelP", "_YvelP", "_ZvelP", "_Pang", "_Hvel", "_Pspd", "_Tdis","_exca","_ctype"];
private ["_discnt","_X","_Y","_Z", "_H", "_Ppos", "_objA", "_SLref", "_GLref", "_ASL", "_types", "_disT", "_objN", "_exct","_extype"];
private ["_dto","_doff","_ang1","_ang2","_maxdis","_return"];
_proj = _this select 0;
_types = _this select 1;
_dis = _this select 2;
_inc = _this select 3;
_maxdis = _this select 4;
_exct = [];
if (count _this > 5) then
	{
	_exct = _this select 5;
	_exca = [];
	{_exca = _exca + [(_x createVehicle [0,0,0])]} forEach _exct;
	_exct = [];
	{_exct = _exct + [(typeOf _x)]} forEach _exca;
	{deleteVehicle _x} forEach _exca;
	};
_exct = _exct + ["SmokeSource","Track","Mark","ObjectDestructed","Explosion","Crater","CraterOnVehicle","Slop","Smoke","DynamicSound"];
_SLref = "emptydetector" createVehicle [0,0,0];
_GLref = "logic" createVehicle [0,0,0];
_objA = [_proj];
_disT = 999999999;
_objN = objNull;

// get the velocity & pos

_Pvel = Velocity _proj;
_Ppos = getPos _proj;
_SLref setPos [_Ppos select 0, _Ppos select 1];
_ASL = _proj distance _SLref;

// get the rectangular velocity vector components

_XvelP = (_Pvel select 0);
_YvelP = (_Pvel select 1);
_ZvelP = (_Pvel select 2);

// find projectile speed and movement direction from velocity vectors
// (this is more reliable than using "getDir" and "Speed" commands)

_Hvel = sqrt(_XvelP^2 + _YvelP^2);
If (_Hvel == 0) then
{
  If (_ZvelP > 0) then {_Pang = 90};
  If (_ZvelP < 0) then {_Pang = -90};
  If (_ZvelP == 0) then {_Pang = 0};
};
If (_Hvel != 0)
then
{
  _Pang = Atan (_ZvelP/_Hvel);
  _Pspd = _Hvel / Cos _Pang;
}
Else
{
  _Pspd = abs _ZvelP;
};

// find compass direction of projectile (movement direction is not necessarily the "getDir" value)

if (_Pspd > 0 && _YvelP != 0) then {_dir = _XvelP atan2 _YvelP} else {_dir = getDir _proj};
if (_dir < 0) then { _dir = _dir + 360 };

// find objects along trajectory path of _proj

_discnt = _inc;
While {_discnt <= _dis} Do
   	{
	_Z = _discnt * Sin _Pang;
	_H = _discnt * Cos _Pang;
	_X = (_Ppos select 0) + (_H * Sin _dir);
	_Y = (_Ppos select 1) + (_H * Cos _dir);
	_GLref setPos [_X, _Y, 0];
	_SLref setPos [_X, _Y];
	_Z = _Z + (_ASL - (_GLref distance _SLref));
	_GLref setPos [0,0,0];
	_SLref setPos [0,0];
	if (_Z < 0) then {_Z = 0};
	_obj = nearestObject [_X, _Y, _Z];
	_extype = false;
	_ctype = false;
	{if (_x countType [_obj] == 1) then {_ctype = true}} forEach _types;
	{if (_x countType [_obj] == 1) then {_extype = true}} forEach _exct;

	if (! (_obj in _objA) && _ctype && ! _extype) then
		{
		// find bearing from projectile to _obj (no targets beyond 45 degrees of the projectile path will be considered)

		if ( ((getpos _obj select 0) - (_Ppos select 0) != 0) || ((getpos _obj select 1) - (_Ppos select 1) != 0) )
		then {_dto = ((getpos _obj select 0) - (_Ppos select 0)) atan2 ((getpos _obj select 1) - (_Ppos select 1))}
		else {_dto = 180};

		// make bearings true compass heading

		if (_dto < 0) then {_dto = _dto + 360};

		// Find Direction Offset

		_ang1 = _dir;
		_ang2 = _dto;
		if ((_ang1 - _ang2) > 180) then {_ang2 = _ang2 + 360};
		if ((_ang2 - _ang1) > 180) then {_ang1 = _ang1 + 360};
		_doff = (_ang2 - _ang1);
		
		if (abs _doff <= 45) then
			{
			_Z = (_proj distance _obj) * Sin _Pang;
			_H = (_proj distance _obj) * Cos _Pang;
			_X = (_Ppos select 0) + (_H * Sin _dir);
			_Y = (_Ppos select 1) + (_H * Cos _dir);
			_GLref setPos [_X, _Y, 0];
			_SLref setPos [_X, _Y];
			_Z = _Z + (_ASL - (_GLref distance _SLref));
			if ("man" countType [_obj] == 1 && alive _obj) then {_Z = _Z - 1}; 
			if ("man" countType [_obj] == 1 && ! alive _obj) then {_Z = _Z + 1.6};
			_GLref setPos [_X, _Y, _Z];
			if (_GLref distance _obj <= _maxdis) then {_objA = _objA + [_obj]};
			if (_GLref distance _obj < _disT && _obj in _objA) then {_disT = _GLref distance _obj; _objN = _obj;};
			};
		};
	_discnt = _discnt + _inc;
	};
deleteVehicle _GLref;
deleteVehicle _SLref;
_objA = _objA - [_proj];
_return = [_objA, _objN, _disT];
if (count _objA == 0) then {_return = []};
_return