/*---------------------------------------------------------------
VelTurn function by Raptorsaurus

Returns the Velocity vectors required to "deflect" an object so that it will
turn toward a given direction.

The required info passed to this function is:

- Object name
- desired direction (0 to 360)
- magnitude of side "defelction"
- angle of side "deflection" (1 to 179)

Example:

_obj = _plane1
_dir = 315
_mag = 5
_ang = 60

_Vel = [_obj,_dir,_mag, _ang] call VelTurn

_obj setVelocity _Vel

This will return the plane1 velocity modified with a side "push" of 5 N (kg m/s^2) above the
the inertia of the plane (so the force will be treated as if the mass of the plane was 1 Kg - hence keep
the force numbers small if you do not want your craft to shift too much).
The "push" or "deflection" will be either to the left or the right relative
to the plane's current heading and the desired direction.  Also the velocity increase in the direction
of push will be as if the force was enacted for a period of 1 second.

Initialize this function by putting this in your init.sqs:
VelTurn = preprocessFile "VelTurn.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_obj","_dir","_mag","_Xvel","_Yvel","_Zvel","_Objdir"];
private ["_ang1","_ang2","_offang","_ang"];

// get the parameters

_obj = _this select 0;
_dir = _this select 1;
_mag = _this select 2;
_ang = _this select 3;

// Make sure deflection angle is positive number

_ang = abs _ang;

_Objdir = getDir _obj;

//Find _obj directional offset from desired direction

_ang1 = _Objdir;
_ang2 = _dir;
if ((_ang1 - _ang2) > 180) then {_ang2 = _ang2 + 360};
if ((_ang2 - _ang1) > 180) then {_ang1 = _ang1 + 360};
_offang = (_ang2 - _ang1);

//set "deflection" angle either to left or right (+90 or -90)

if (_offang < 0) then {_ang = -_ang};

//Find directional offset between _obj heading and "deflection" angle (_ang)

_ang1 = _Objdir;
_ang2 = _ang;
if ((_ang1 - _ang2) > 180) then {_ang2 = _ang2 + 360};
if ((_ang2 - _ang1) > 180) then {_ang1 = _ang1 + 360};
_ang = (_ang1 + _ang2);
if (_ang > 360) then {_ang = _ang - 360};
if (_ang < 0) then {_ang = _ang + 360};

// get velocity components

_Xvel = velocity _obj select 0;
_Yvel = velocity _obj select 1;
_Zvel = velocity _obj select 2;

// calculate modified velocity components in horizontal plane

_Xvel = _Xvel + (_mag * Sin _ang);
_Yvel = _Yvel + (_mag * Cos _ang);

// new velocity vector array

[_Xvel,_Yvel,_Zvel]