/*---------------------------------------------------------------
VeltoAngDir function by Raptorsaurus

Returns the Velocity vectors required to alter a moving objects trajectory so
that it will climb or decend at a given angle, and move in a given direction (compass)
while maintaining its current speed (speed can be forced using corection factor, min &
max speed parameters).

Note: THE OBJECT MUST BE MOVING & ITS SPEED WILL REMAIN CONSTANT UNLESS A
            CORRECTION FACTOR AND MIN AND MAX GREATER THAN 0 IS ENTERED.

The required info passed to this function is:

- Object name
- desired angle (positive to climb, negative to decend)
- desired compass heading
- Speed correction factor (perhaps for gravity or wind resistance or both, put zero for no effect, [value in m/s] )
- minimum speed (to limit object from slowing too much, put zero for no effect [value in m/s])
- maximum speed (to prevent object from exceeding its terminal velocity, put zero for no effect [value in m/s])

Example:

_obj = _missile
_ang = 35
_dir = 135
_cor = -.109 * _ang * _reftime (at -90 deg. the _obj will accelerate at 9.81 m/s/s [gravity])
_minspeed
_maxspeed

_Vel = [_obj,_ang,_dir,_cor,_min,_max] call VeltoAngDir

_obj setVelocity _Vel

This will alter the missiles velocity so it will be climbing at 35 degrees
relative to horizontal and move at 135 deg.  Since some ojects in OFP (like missiles)
generate their own thrust and are subject to gravity and air drag the correction
factor may be used.  Also, since it is unrealistic for a moving object to instantaneously
change its velocity this function should be used in loop that will incrementally
alter the objects direction (i.e. if a missile is traveling at a 10 degree climb
it would look really silly if it suddenly went straight up.  So a loop could be
used to send the angle to this function in small increments.

Initialize this function by putting this in your init.sqs:
VeltoAngDir = preprocessFile "VeltoAngDir.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_obj","_dir","_ang","_Hvel","_Spd","_Xvel","_Yvel","_Zvel"];
private ["_x","_y","_z","_tang","_cor","_min","_max"];

_obj = _this select 0;
_ang = _this select 1;
_dir = _this select 2;
_cor = _this select 3;
_min = _this select 4;
_max = _this select 5;

_vel = Velocity _obj;

// get the rectangular velocity vector components

_x = _vel select 0;
_y = _vel select 1;
_z = _vel select 2;

// find speed from velocity vectors

_Hvel = sqrt(_x^2 + _y^2);

If (_Hvel != 0)
then
{
  _tang = Atan (_z/_Hvel);
  _Spd = _Hvel / Cos _tang;
}
Else
{
  If (_z > 0) then {_tang = 90};
  If (_z < 0) then {_tang = -90};
  If (_z == 0) then {_tang = 0};
  _Spd = abs _z;
};

//modify the _Spd in accordance with the correction factor & min/max speed)

_Spd = _Spd + _cor;
if (_Spd < _min && _min > 0) then {_Spd = _min};
if (_Spd > _max && _max > 0) then {_Spd = _max};

// calculate the projected horizontal velocity component at current speed and desired _ang

_Hvel = _Spd * Cos _ang;

// calculate projected velocity components in horizontal plane

_Xvel = _Hvel * Sin _dir;
_Yvel = _Hvel * Cos _dir;

// calculate vertical velocity component

_Zvel = _Spd * Sin _ang;

// new velocity array to set object in dive/climb in desired direction

[_Xvel,_Yvel,_Zvel]