/*---------------------------------------------------------------
converge function by Raptorsaurus

Returns the velocity required to make one vehicle converge upon the position of another vehicle or
an offset point from the other vehicle. Thus this can be used in maintianing aircraft formations.

Pass the target vehicle, chase vehicle, max relative convergence speed, min relative convergence speed,
start convergence distance, end convergence distance, maximum absolute speed, minimum absolute speed and
relative convergence point position offset from the target vehicle.

Example:

_lead = Lead aircraft of formation (for example)
_chase = aircraft to form on lead
_CSmax = the speed at which the chaser will overtake the lead (until is cathces up)
_CSmin = the speed the chaser will slow to (relative to lead) as it reaches the convergence distance
_CDmax = the distance at which the convergence speed will begin to slow (relative to lead speed)
_CDmin = the distance at which the convergence speed will reach the _CSmin value
_Smax = the absolute maximum speed of the chaser (so it will not ever go unrealistically fast)
_Smin = the absolute minimum speed of the chaser (so it will not ever go too slow).
_pOff = the offset point relative to the lead aircraft where the chaser should "form up". Note: the
	  convergence distances will always be the chasers distance to this point, NOT the distance to
	  the lead aircraft.

All speeds are in km/h and all distances are in meters.

_vel = [_lead,_chase,_CSmax,_CSmin,_CDmax,_CDmin, _Smax, _Smin, _pOff] call converge

or

_vel = [C130, A10, 100, 0, 150, 0, 400, 150, [50,-50,20]] call convergeSpd

Used in a loop with other functions (as mentioned above).  This will return
the velocity required to have the A10 move to a formation point 50 m to the right of
the C130, 50 m behind, and 20 m above the C130. The A10 will converge to that point
at a maximum speed of 100 km/h relative to the C130. It will start to equalize velocity
with the C130 when it is 150 m from the formation point. When it is exactly at the formation
point it will have the same relative velocity as the C130. The A10 will never exceed 400 km/h
and will never go under 150 km/h.

Initialize this function by putting this in your init.sqs:
converge = preprocessFile "converge.sqf"
---------------------------------------------------------------*/

Private ["_lead","_chase","_CSmax","_CSmin","_CDmax","_CDmin","_Speed","_b","_dis","_m","_poff","_fdir","_doff","_Tint"];
private ["_Refpos","_x","_y","_z","_Refdir","_offFB","_offLR","_offAB","_leadX","_leadY","_Zvel","_ang2","_Lspd","_cpnt"];
private ["_leewX","_leewY","_SLref","_TPref","_alt1","_alt2","_Hvel","_ang","_setspd","_Smax","_Smin","_ang1","_Ldis"];
_lead = _this select 0;
_chase = _this select 1;
_CSmax = _this select 2;
_CSmin = _this select 3;
_CDmax = _this select 4;
_CDmin = _this select 5;
_Smax = _this select 6;
_Smin = _this select 7;
_poff = _this select 8;

//Create detectors use for finding altitude and distance offsets.

_SLref = "emptyDetector" createVehicle [0,0,0];
_TPref = "logic" createVehicle [0,0,0];

//Determine true speed of lead aircraft

_Hvel = sqrt((velocity _lead select 0)^2 + (velocity _lead select 1)^2);
_Zvel = (velocity _lead select 2);
If (_Hvel == 0) then
{
  If (_Zvel > 0) then {_ang = 90};
  If (_Zvel < 0) then {_ang = -90};
  If (_Zvel == 0) then {_ang = 0};
};
If (_Hvel != 0)
then
{
  _ang = Atan (_Zvel/_Hvel);
  _Lspd = (_Hvel / Cos _ang) * 3600 / 1000;
}
Else
{
  _Lspd = (abs _Zvel) * 3600 / 1000;
};

//Find the absolute position of the convergence point from the relative offsets and determine the chase distanc to that point

_offLR = _poff select 0;
_offFB = _poff select 1;
_offAB = _poff select 2;
_Refdir = getDir _lead;
_Refpos = getPos _lead;
_x = _Refpos select 0;
_y = _Refpos select 1;
_SLref setPos [_x, _y];
_TPref setPos [_x, _y, 0];
_alt1 = _TPref distance _SLref;
_leadX = (_offFB * Sin _Refdir);
_leadY = (_offFB * Cos _Refdir);
_leewX = (_offLR * Sin (_Refdir + 90));
_leewY = (_offLR * Cos (_Refdir + 90));
_SLref setPos [(_leadX + _leewX + _x),(_leadY + _leewY + _y)];
_TPref setPos [(_leadX + _leewX + _x),(_leadY + _leewY + _y), 0];
_alt2 = _TPref distance _SLref;
_offAB = _offAB + (_alt1 - _alt2);
_z = (_Refpos select 2) + _offAB;
_cpnt = [(_leadX + _leewX + _x),(_leadY + _leewY + _y),_z];
_TPref setPos _cpnt;
_dis = _chase distance _TPref;

//Determine the true direction of the chase aircraft (movement direction if it is moving, getDir if not)

_x = (velocity _chase select 0);
_y = (velocity _chase select 1);
if (_y == 0) then {_y = .0000001}; 
_Hvel = sqrt(_x^2 + _y^2);
if (_Hvel != 0) then {_refdir = _x atan2 _y} else {_refdir = getDir _chase};
if (_refdir < 0) then {_refdir = _refdir + 360};

//Determine the relative direction to the formation point from the chase aircraft's position

_x = (getpos _TPref select 0) - (getpos _chase select 0);
_y = (getpos _TPref select 1) - (getpos _chase select 1);
if (_y == 0) then {_y = .0000001};
_fdir = _x atan2 _y;
if (_fdir < 0) then {_fdir = _fdir + 360};
_ang1 = _refdir;
_ang2 = _fdir;
if ((_ang1 - _ang2) > 180) then {_ang2 = _ang2 + 360};
if ((_ang2 - _ang1) > 180) then {_ang1 = _ang1 + 360};
_doff = (_ang2 - _ang1);

//Determine chase setspeed as it converges with the formation point

if (_dis <= _CDmin) then {_CDmax = _CDmin; _CSmax = _CSmin; _CDmin = 0; _CSmin = 0};
_m = (_CSmax - _CSmin) / (_CDmax - _CDmin);
_b = _CSmin - (_m * _CDmin);
_Speed = _CSmax;
if (_dis <= _CDmax) then {_Speed = (_m * _dis) + _b};
_setspd = abs (_speed + (_Lspd * cos _doff));

//Make sure the setspeed will not make the chase exceed its speed limits

If (_setspd > _Smax) then {_setspd = _Smax};
If (_setspd < _Smin) then {_setspd = _Smin};

//convert setspeed and speed to m/s for velocity calculations

_setspd = _setspd * 1000 / 3600;
_Speed = _Speed * 1000 / 3600;

//calculation the theoretical time to reach the formation point

if (_Speed == 0) then {_Speed = .0000001};
_Tint = abs (_dis / _Speed);

//calculate chase velocity to make it converge with the formation point

_x = (_Tint * (velocity _lead select 0)) + (_cpnt select 0);
_y = (_Tint * (velocity _lead select 1)) + (_cpnt select 1);
_z = (_Tint * (velocity _lead select 2)) + (_cpnt select 2);
_x = _x - (getPos _chase select 0);
_y = _y - (getPos _chase select 1);
_z = _z - (getPos _chase select 2);
if (_x == 0) then {_x = .0000001};
if (_y == 0) then {_y = .0000001};
_fdir = _x Atan2 _y;
if (_fdir < 0) then { _fdir = _fdir + 360 };
_dis = sqrt (_x^2 + _y^2);
if (_dis == 0) then {_dis = .0000001};
_Rang = Atan (_z / _dis);
_RHvel = _setspd * Cos _Rang;
_x = _RHvel * Sin _fdir;
_y = _RHvel * Cos _fdir;
_z = _setspd * Sin _Rang;

//delete the created detectors

deleteVehicle _SLref;
deleteVehicle _TPref;

//return the calculated velocity
[_x,_y,_z]