/*---------------------------------------------------------------
convergeSpd function by Raptorsaurus

Returns the speed for a vehicle adjusted for distance to another vehicle it is
to converge with or slowly pass etc.  This function can be used in a loop along with
the functions: Intercept , VeltoAng and SetSpd to make aircraft formations etc.

Pass the target vehicle, chase vehicle, max convergence speed, min convergence speed,
start convergence distance and end convergence distacne

Example:

_lead = Lead aircraft of formation (for example)
_chase = aircraft to form on lead
_CSmax = the speed at which the chaser will overtake the lead (until is cathces up)
_CSmin = the speed the chaser will slow to (relative to lead) as it reaches the convergence distance
_CDmax = the distance at which the convergence speed will begin to slow (relative to lead speed)
_CDmin = the distance at which the convergence speed will reach the _CSmin value

_spd = [_lead,_chase,_CSmax,_CSmin,_CDmax,_CDmin] call convergeSpd

or

_spd = [C130,A10,100,0,150,30] call convergeSpd

Used in a loop with other functions (as mentioned above).  This will return
the speed required to have the A10 catch up to the C130 (it will initially go
100 k/h faster than the C130 -- the _CSmax speed).  When its distance to the C130
is less than 150 m (the _CDmax) it will begin to slow to the same speed as the C130
(proportionally to it closing distance).  When it closes to 30 m (the _CDmin) its
speed will be matched to that of the C130 (the _CSmin).

When the A10 is more the 150 m away return speed value will be the C130 speed + 100.
At 100 m away the return speed value will be the C130 speed + 58.333.  At 50 m
away speed will be C130 speed + 16.667.  At 30 m away it will have the same speed
as the C130.  If it gets closer than 30 m the returned speed will be less than that
of the C130 so it will fall back to its _CDmin distance.

Initialize this function by putting this in your init.sqs:
convergeSpd = preprocessFile "convergeSpd.sqf"
---------------------------------------------------------------*/

Private ["_lead","_chase","_CSmax","_CSmin","_CDmax","_CDmin","_Speed","_b","_dis","_leadspd","_m"];

// Get info

_lead = _this select 0;
_chase = _this select 1;
_CSmax = _this select 2;
_CSmin = _this select 3;
_CDmax = _this select 4;
_CDmin = _this select 5;

_leadspd = Speed _lead;

// Find _b and _m value ("b" and "m" in linear equation: y=mx+b)

_m = (_CSmax - _CSmin)/(_CDmax - _CDmin);
_b = _CSmin - (_m * _CDmin) ;

// Find distance between _lead and _chase

_dis = _lead Distance _chase;

// Caluculate speed adjusting for closing distance

_Speed = (_m * _dis) + _b;

// if distance is greater than _CDmax then _Speed = _CSmax

If (_dis > _CDmax) then {_Speed = _CSmax};

//return values

_Speed + _leadspd