/*---------------------------------------------------------------
Equipment function by Raptorsaurus
v. 1.0, 8/25/05

Returns an array containing the name of all the equipment caried by a unit and the magazines for each weapon.  Optionally, one can also use this
function to switch the unit's present weapon loadout.

The required info passed to this function is:

[unit, ["pweap", "sweap", "tweap"], [["pmag", 4], ["smag", 3], ["tmag", 4], ["HandGrenade", 3]], ["NVGoggles", "Binocular"]] call Equipment

- unit : name of unit
- OPTIONAL -->  ["pweap", "sweap", "tweap"] : Array of all weapons to give to the man. This array must have three elements or be empty. The
		first is the primary weapon, the second it the secondary weapon and the third is the sidearm. If you do not want to change a
		weapon then put an empty string in place of that weapon (eg. to keep existing primary weapon, but change others put -->
		["", "LAWlauncher", "Beretta"]. To not change any weapons make the array empty --> [].
- OPTIONAL -->  [["pmag", 4], ["smag", 3], ["tmag", 4], ["HandGrenade", 3]] : Array of magazines and the quantity for each magazine. This is
		in "array of arrays" format. Each sub-array is the name of the magazine and the quantity for that magazine. The first sub-array
		must be the primary weapon magazine and quantity, the second must be the secondary weapon magazine and quantity and
		the third must be the sidearm weapon magazine and quantity. Any thrown or put magzine arrays (after these first three) can
		can be in any order. If you do not want to change magazines for a weapon then put an empty array in the position appropriate
		for that weapon. eg. [[], [], [glockSmag, 4], []] will not effect the existing primary, secondary or thrown weapon magazines. Only the
		the sidearm magazines will be changed. To remove magzines use and empty string "" and quantity of zero ["", 0] for the magazine
		group to be removed. eg. [["",0],["",0], [], []] will remove the primary magazines and secondary magazines while leaving handgun and
		thrown magazines unchanged.
- OPTIONAL -->  ["NVGoggles", "Binocular"] : Array of optical equipment. To not effect existing equipment leave this parameter out. To remove all
		optical equipment enter an empty array []. If you enter a single element array, then only that piece of equipment will be present.

Notes:	Be careful not to give units invalid combinations of weapons/magazines and invalid quantities thereof.  A man cannot have 10
	handgrenades and also 10 M16 magazines.

	The primary, secondary and sidearm magazine test arrays (see comments below) must contain all the possible magazine types that a unit
	in your mission might have. If you are using addon weapons make sure you add those weapon magazine names to the correct test array if
	they are not allready in an array. The magazines used in a primary weapon's secondary muzzel (like grenade launchers) should be included in
	the test array for primary magazines (this would include flares). WHEN ADDING A MAGAZINE TO AN ARRAY BE AWARE THAT MAGAZINE
	NAMES ARE CASE SENSITIVE. If a magazine name is not properly included in the test arrays, then that magazine will be listed among the
	array of thrown and put weapons. You can use this feature to update the test array as needed.

Example 1:

_equipinfo = [_myguy] call Equipment

_primary = _equipinfo select 0	(This will be a two dim array containing the primary weapon (the first element) and a sub-array of all the
				magazines for that weapon. If he has none it will be an empty array.)
_secdary = _equipinfo select 1 	(This will be a two dim array containing the secondary weapon (the first element) and a sub-array of all the
				magazines for that weapon. If he has none it will be an empty array.)
_tertiary = _equipinfo select 2 	(This will be a two dim array containing the sidearm weapon (the first element) and a sub-array of all the
				magazines for that weapon. If he has none it will be an empty array.)
_thrown = _equipinfo select 3 	(This will be an array containing the name of all the thrown and put magazines carried by _myguy.
				If he has none it will be an empty array.)
_optical = _equipinfo select 4 	(This will be an array containing the name of the optical equipment carried by _myguy. If he has none
				it will be an empty array.)

Example 2:

_oldequip = [_myguy, ["Bizon", "", "glockS"], [["BizonMag", 6],["",0], ["glockSmag", 4], ["handgrenade",4]], ["binocular", "NVgoggles"] ] call Equipment

This will give a Bizon, no secondary weapon and a glockS to myguy. It will give him 6 magazines for the Bizon, 4 glockS mags and 4 handgrenades.
He will also be given binoculars and NVgoggles. The _oldequip will be an array of the stuff he had before his weapons and ammo was changed.

Example 3:

_oldequip = [_myguy, ["AK47", "AT4Launcher", ""], [["AK47", 4],["AT4Launcher",1], ["", 0], ["",0]], ["binocular", "NVgoggles"] ] call Equipment

This will give an AK47, and AT4Launcher and no handgun to myguy. It will give him 4 magazines for the AK47, 1 AT4 Missile and remove all handgun magazines
and handgrenades (in case he had some that might block the addition of the AT4).
He will also be given binoculars and NVgoggles. The _oldequip will be an array of the stuff he had before his weapons and ammo was changed.

Initialize this function by putting this in your init.sqs:
Equipment = preprocessFile "Equipment.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_man","_wps","_mags","_testwps","_pw", "_sw","_tw","_pmags","_smags","_tmags","_hmags","_eq"];
private ["_amags","_mtyp","_mcnt","_primMags", "_secMags", "_sidearmMags"];

_man = _this select 0;

_pm = "";
_sm = "";
_tw = "";
_tm = "";
_pmags = [];
_smags = [];
_tmags = [];
_hmags = [];
_eq = [];
_wps = weapons _man;
_pw = primaryWeapon _man;
_sw = secondaryWeapon _man;
_mags = magazines _man;
_testwps = [_pw, _sw, "NVGoggles", "Binocular"];

// These are test arrays.  The possilbe magazine types that a unit might have at the time this function is called, MUST be in the appropriate array below.
_primMags = ["AK74","BizonMag","FALMag","G36AMag","HK","HKG3Mag","HuntingRifleMag","JahSG551mag","JAM_MP5SBASDmag","JAM_W556M_200SDmag"];
_primMags = _primMags + ["JAM_W556_30SDmag","KozliceBall","KozliceShell","M16","M21","M4","M60","PK","SteyrMag","SVDDragunov","GrenadeLauncher"];
_primMags = _primMags + ["MM1Magazine","Mortar","UZIMag","Flare","FlareGreen","FlareRed","FlareYellow","JAM_MarkerGrenades","JAM_flareWhite"];
_primMags = _primMags + ["JAM_M433Vest"];
_secMags = ["9K32Launcher","AALauncher","AT4Launcher","CarlGustavLauncher","LAWLauncher","LaserDesignator","RPGLauncher","JAM_AT4Rocket","BAS_AT4Rocket"];
_sidearmMags = ["BerettaMag","CZ75Mag","GlockMag","GlockSMag","IngramMag","JAM_GlockSDmag","RevolverMag","SDOC_HKMP5Kmag","SkorpionMag","TokarevMag"];

// find the sidearm (if any) among the _man's weapons

if (count (_wps - _testwps) > 0) then {_tw = (_wps - _testwps) select 0};

//determine primary weapon mags

{ if (_x in _primMags) then {_pmags = _pmags + [_x]} } forEach _mags;

//determine secondary weapon mags

{ if (_x in _secMags) then {_smags = _smags + [_x]} } forEach _mags;

//determine sidearm weapon mags

{ if (_x in _sidearmMags) then {_tmags = _tmags + [_x]} } forEach _mags;

//determine the thrown/put/misc weapons

_hmags = _mags - (_pmags + _smags + _tmags);

//determine the optical equipment

_eq = _wps - [_pw, _sw, _tw];

//remove old weapons

if (count _this > 1) then
	{
	if (count (_this select 1) > 0) then
		{
		if ((_this select 1) select 0 != "") then {_man removeWeapon _pw};
		if ((_this select 1) select 1 != "") then {_man removeWeapon _sw};
		if ((_this select 1) select 2 != "") then {_man removeWeapon _tw};
		};
	
	if (count (_this select 2) > 0) then
		{
		_amags = _this select 2;

//remove old magazines

		if (count (_amags select 0) > 0) then {{_man removeMagazine _x} forEach _pmags};
		if (count (_amags select 1) > 0) then {{_man removeMagazine _x} forEach _smags};
		if (count (_amags select 2) > 0) then {{_man removeMagazine _x} forEach _tmags};
		if (count _amags > 3 && count (_amags select 3) > 0) then {{_man removeMagazine _x} forEach _hmags};
		
//add replacement magazines

		if (count (_amags select 0) > 0) then 
			{
			_mtyp = (_amags select 0) select 0;
			_cnt = (_amags select 0) select 1;
			While {_cnt > 0} do
				{
				_man addMagazine _mtyp;
				_cnt = _cnt - 1;
				};
			};
		if (count (_amags select 1) > 0) then 
			{
			_mtyp = (_amags select 1) select 0;
			_cnt = (_amags select 1) select 1;
			While {_cnt > 0} do
				{
				_man addMagazine _mtyp;
				_cnt = _cnt - 1;
				};
			};
		if (count (_amags select 2) > 0) then 
			{
			_mtyp = (_amags select 2) select 0;
			_cnt = (_amags select 2) select 1;
			While {_cnt > 0} do
				{
				_man addMagazine _mtyp;
				_cnt = _cnt - 1;
				};
			};
		if  (count _amags > 3) then
			{
			_cnt = (count _amags) - 1;
			While {_cnt > 2} do
				{
				_mtyp = (_amags select _cnt) select 0;
				_mcnt = (_amags select _cnt) select 1;
				While {_mcnt > 0} do
					{
					_man addMagazine _mtyp;
					_mcnt = _mcnt - 1;
					};
				_cnt = _cnt - 1;
				};
			};
		};
	};

//add replacement weapons & optical equipment

if (count _this > 1) then
	{
	if (count (_this select 1) > 0) then { { if (_x != "") then {_man addWeapon _x} } forEach (_this select 1) };
	if (count _this > 3) then
		{
		{_man RemoveWeapon _x} forEach _eq; 
		if (count (_this select 3) > 0) then {{_man addWeapon _x} forEach (_this select 3)};
		};
	_man action ["handgun on"];
	_man action ["handgun off"];
	};

//_man's original equipment

[[_pw, _pmags], [_sw, _smags], [_tw, _tmags], _hmags, _eq]