/*---------------------------------------------------------------
gravOnActProj function by Raptorsaurus

Adjusts the velocity vectors of active projectile (one which has sustained power)
so that it speeds or slows proportionally to its angle of climb or dive in
accordance with gravitational acceleration.  It is assumed that the missile has
a constant level of thrust and so its angle of climb/dive will be maintained
(unlike a passive projectile - a missile is active, a cannon shell is passive).

Pass info to the function in the following format:

[_mis,_reftime,_minspeed,_maxspeed] call gravOnActProj

The _reftime (reference time) value must be included in a loop that uses this function.
The _reftime value should be updated right after this function is called, so that
the delta time until the loop gets back to this function will be accurate.  The _reftime
should also be initialize before the loop begins.
The _minspeed and _maxspeed are limits for terminal velocity (_max speed do to air friction) and
vertical climb rate (a missile with thrust that exceeds its weight will never be slowed
below a minimum speed).

Initialize this function by putting this in your init.sqs:
gravOnActProj = preprocessFile "gravOnActProj.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_mis","_reftime","_x","_y","_z","_Hvel","_Ang","_Mag","_dir","_gafct"];
private ["_minspeed","_maxspeed","_speed"];

// get needed values

_mis = _this select 0;
_reftime = _this select 1;
_minspeed = _this select 2;
_maxspeed = _this select 3;

_speed = Speed _mis;

While {_speed > _minspeed && _speed < _maxspeed}
Do
{
   _vel = Velocity _mis;
   _dir = getDir _mis;

   // get the rectangular velocity vector components

   _x = _vel select 0;
   _y = _vel select 1;
   _z = _vel select 2;

   // convert to magnitued & angle format

   _Hvel = sqrt(_x^2 + _y^2);

   If (_Hvel == 0) then
      {
      If (_z > 0) then {_Ang = 90};
      If (_z < 0) then {_Ang = -90};
      If (_z == 0) then {_Ang = 0};
      };
   If (_Hvel != 0) then
      {
      _Ang = Atan (_z/_Hvel);
      _Mag = _Hvel / Cos _Ang;
      }
     Else
      {
      _Mag = abs _z;
      };

   // get motion based direction

   If (_Hvel > 0) then {_dir = _x Atan2 _y};
   If (_dir < 360) then {_dir = _dir + 360};

   // calculate gfactor (.109 m/s/s/deg [9.81 / 90 deg = .109])

   _gfact = -.109 * _Ang;

   //modify the _Mag in accordance with gravity and the elapsed time since
   //_reftime.

   _Mag = _Mag + (_gfact * (time - _reftime));

   // modify _Hvel & _z to maintain the the same angle with the new _Mag value.

   _Hvel = _Mag * Cos _Ang;
   _z = _Mag * Sin _Ang;

   _x = _Hvel * Sin _dir;
   _y = _Hvel * Cos _dir;
};

// return the updated _mis velocity
[_x,_y,_z];