/*---------------------------------------------------------------
setMkLowAlt function by Raptorsaurus

Places a marker at the lowest altitude possible within a target area.  The
target area is defined by the line formed by the marker and two other
reference markers, the permissible perpendicular deviation from the line and
the search increment used to find the low spot within the deviation range.  This
function is used to optimize the NOE flight path of an aircraft using markers as
waypoints.

Pass info to the function in the following format:

[_RefMk1,_RefMk2,_PlcMk,_Dev,_Inc,_DeltaAlt] call setMkLowAlt

or

["RefMarker1","RefMarker2","MarkertoPosition",250,10,100] call setMkLowAlt

The RefMarkers determine the optimum straight line flight path, the
MarkertoPosition is the marker that is being adjusted to the low altitude
position.  The 100 is the allowable positive altitude change before the the
function determines that a "valley has been found (keeps waypoint from going over
a mountain or ridge to find a lower spot), the 250 is the permissible linear
deviation perpendicular to the flight path, the 10 is the test point increments-
with a plus/minus deviation of 250 and increments of 10, 25 points will be
tested in both directions perpendicular to the flight path (50 total).  If the
Delta alt (+100) is reached before all the points are tested the function will
take the lowest point found before the delta alt was reached.

Initialize this function by putting this in your init.sqs:
setMkLowAlt = preprocessFile "setMkLowAlt.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_posRef1","_posRef2","_posMk","_Mk","_dir1","_dir2","_dir","_Dev"];
private ["_SLref","_Tref","_Dis","_Inc","_cnt","_RecPos","_x","_y","_Dalt"];
private ["_xTref","_yTref","_posTref","_Ang","_ASL","_Hdis","_RecASL"];

//get info

_posRef1 = getMarkerPos (_this select 0);
_posRef2 = getMarkerPos (_this select 1);
_posMk = getMarkerPos (_this select 2);
_Mk = _this select 2;
_Dev = _this select 3;
_Inc = _this select 4;
_Dalt = _this select 5;

_cnt = _Inc;
_RecPos = _posMk;

// get x & y coordinates from refMarker

_x = _posMk select 0;
_y = _posMk select 1;

// calculate direction (compass heading)

_dir = ((_posRef2 select 0) - (_posRef1 select 0)) atan2 ((_posRef2 select 1) - (_posRef1 select 1));
if (_dir < 0) then {_dir = _dir + 360};

// find perpendicular headings

_dir1=_dir + 90;
if (_dir1 > 360) then {_dir1 = _dir1 - 360};
_dir2=_dir - 90;
if (_dir2 < 360) then {_dir2 = _dir2 + 360};

// Create Sea level reference and Test reference

_SLref = "logic" camCreate [0,0,0];
_Tref = "logic" camCreate [0,0,0];

// Check ASL at current _posMk

_Tref setPos [_x,_y];

// get the position

   _PosTref = getPos _Tref;
   _xTref = _PosTref select 0;
   _yTref = _PosTref select 1;

   //Calculate ASL

   _Hdis = sqrt(_xTref^2 + _yTref^2);
   _Dis = _Tref distance _SLref;
   _Ang = Acos (_Hdis/_Dis);
   _ASL = _Hdis * Tan _Ang;

// Place _Tref at incremental points along test axis, record the positions if
// they are lower than the previously recorded position.

While {_cnt < _Dev && _ASL > 20}
Do
{
   // Place _Tref at positive angle and get ASL
   _Tref setPos [(_x + (_cnt * Sin _dir1)),(_y + (_cnt * Cos _dir1))];

   // get the position

   _PosTref = getPos _Tref;
   _xTref = _PosTref select 0;
   _yTref = _PosTref select 1;

   //Calculate ASL

   _Hdis = sqrt(_xTref^2 + _yTref^2);
   _Dis = _Tref distance _SLref;
   _Ang = Acos (_Hdis/_Dis);
   _ASL = _Hdis * Tan _Ang;

   // If _ASL is lower than the last recorded ALS then record the ASL and position

   If (_ASL < _RecASL) then
      {
      _RecASL = _ASL;
      _RecPos = _PosTref;
      };

   // If _Dalt is reached then end function

   If (_ASL - _RecASL > _Dalt) then {_cnt = _Dev};

   // Place _Tref at negative angle and get ASL
   _Tref setPos [(_x + (_cnt * Sin _dir2)),(_y + (_cnt * Cos _dir2))];

   // get the position

   _PosTref = getPos _Tref;
   _xTref = _PosTref select 0;
   _yTref = _PosTref select 1;

   //Calculate ASL

   _Hdis = sqrt(_xTref^2 + _yTref^2);
   _Dis = _Tref distance _SLref;
   _Ang = Acos (_Hdis/_Dis);
   _ASL = _Hdis * Tan _Ang;

   // If _ASL is lower than the last recorded ASL then record the ASL and position

   If (_ASL < _RecASL) then
      {
      _RecASL = _ASL;
      _RecPos = _PosTref;
      };

   // If _Dalt is reached then end function

   If (_ASL - _RecASL > _Dalt) then {_cnt = _Dev};
   
   _cnt = _cnt + _Inc;
};

// Set the marker at the lowest recorded position

_Mk setMarkerPos _RecPos;
