/*---------------------------------------------------------------
setObLowAlt function by Raptorsaurus

Places an object at the lowest altitude posible within a target area.  The
target area is defined by the line formed by the object and two other
reference objects, the permissible perpendicular deviation from the line and
the search increment used to find the low spot within the deviation range. This
function is used to optimize the NOE flight path of an aircraft using objects as
waypoints.

Pass info to the function in the following format:

[_RefOb1,_RefOb2,_PlcOb,_Dev,_Inc,_DeltaAlt] call setObLowAlt

The _RefOb's determine the optimum straight line flight path, the
_PlcOb is the object that is being adjusted to the low altitude
position.  The _DeltaAlt is the allowable positive altitude change before the the
function determines that a "valley has been found (keeps waypoint from going over
a mountain or ridge to find a lower spot), the _Dev is the permissible linear
deviation perpindicular to the flight path, the _Inc is the test point increments.
If the plus/minus deviation is 250 and increments are 10 then 25 points will be
tested in both directions perpendicular to the flight path (50 total).  If the
_DeltaAlt is reached before all the points are tested the function will
take the lowest point found before the delta alt was reached.

Initialize this function by putting this in your init.sqs:
setObLowAlt = preprocessFile "setObLowAlt.sqf"
---------------------------------------------------------------*/

// declare private variables
private ["_posRef1","_posRef2","_posOb","_Ob","_dir1","_dir2","_dir","_Dev"];
private ["_SLref","_Tref","_Dis","_Inc","_cnt","_RecPos","_x","_y","_Dalt"];
private ["_xTref","_yTref","_posTref","_Ang","_ASL","_Hdis","_RecASL"];

//get info

_posRef1 = getPos (_this select 0);
_posRef2 = getPos (_this select 1);
_posOb = getPos (_this select 2);
_Ob = _this select 2;
_Dev = _this select 3;
_Inc = _this select 4;
_Dalt = _this select 5;

_cnt = _Inc;
_RecPos = _posOb;

// get x & y coordinates from refObject

_x = _posOb select 0;
_y = _posOb select 1;

// calculate direction (compass heading)

_dir = ((_posRef2 select 0) - (_posRef1 select 0)) atan2 ((_posRef2 select 1) - (_posRef1 select 1));
if (_dir < 0) then {_dir = _dir + 360};

// find perpendicular headings

_dir1=_dir + 90;
if (_dir1 > 360) then {_dir1 = _dir1 - 360};
_dir2=_dir - 90;
if (_dir2 < 360) then {_dir2 = _dir2 + 360};

// Create Sea level reference and Test reference

_SLref = "logic" camCreate [0,0,0];
_Tref = "logic" camCreate [0,0,0];

// Check ASL at current _posOb

_Tref setPos [_x,_y];

// get the position

   _PosTref = getPos _Tref;
   _xTref = _PosTref select 0;
   _yTref = _PosTref select 1;

   //Calculate ASL

   _Hdis = sqrt(_xTref^2 + _yTref^2);
   _Dis = _Tref distance _SLref;
   _Ang = Acos (_Hdis/_Dis);
   _ASL = _Hdis * Tan _Ang;

// Place _Tref at incremental points along test axis, record the positions if
// they are lower than the previously recorded position.

While {_cnt < _Dev && _ASL > 20}
Do
{
   // Place _Tref at positive angle and get ASL
   _Tref setPos [(_x + (_cnt * Sin _dir1)),(_y + (_cnt * Cos _dir1))];

   // get the position

   _PosTref = getPos _Tref;
   _xTref = _PosTref select 0;
   _yTref = _PosTref select 1;

   //Calculate ASL

   _Hdis = sqrt(_xTref^2 + _yTref^2);
   _Dis = _Tref distance _SLref;
   _Ang = Acos (_Hdis/_Dis);
   _ASL = _Hdis * Tan _Ang;

   // If _ASL is lower than the last recorded ALS then record the ASL and position

   If (_ASL < _RecASL) then
      {
      _RecASL = _ASL;
      _RecPos = _PosTref;
      };

   // If _Dalt is reached then end function

   If (_ASL - _RecASL > _Dalt) then {_cnt = _Dev};

   // Place _Tref at negative angle and get ASL
   _Tref setPos [(_x + (_cnt * Sin _dir2)),(_y + (_cnt * Cos _dir2))];

   // get the position

   _PosTref = getPos _Tref;
   _xTref = _PosTref select 0;
   _yTref = _PosTref select 1;

   //Calculate ASL

   _Hdis = sqrt(_xTref^2 + _yTref^2);
   _Dis = _Tref distance _SLref;
   _Ang = Acos (_Hdis/_Dis);
   _ASL = _Hdis * Tan _Ang;

   // If _ASL is lower than the last recorded ASL then record the ASL and position

   If (_ASL < _RecASL) then
      {
      _RecASL = _ASL;
      _RecPos = _PosTref;
      };

   // If _Dalt is reached then end function

   If (_ASL - _RecASL > _Dalt) then {_cnt = _Dev};
   
   _cnt = _cnt + _Inc;
};

// Set the object at the lowest recorded position

_Ob setPos _RecPos;
